@@ -3,7 +3,6 @@ package commands
 import (
 	"fmt"
 	"net/http"
-	"regexp"
 	"strings"
 	"sync"
 
@@ -45,8 +44,8 @@ type statePermutation struct {
 func newStatePermutationFromBuild(build *common.Build) statePermutation {
 	return statePermutation{
 		runner:        build.Runner.ShortDescription(),
-		buildState:    build.CurrentState,
-		buildStage:    build.CurrentStage,
+		buildState:    build.CurrentState(),
+		buildStage:    build.CurrentStage(),
 		executorStage: build.CurrentExecutorStage(),
 	}
 }
@@ -220,10 +219,24 @@ func (b *buildsHelper) statesAndStages() map[statePermutation]int {
 	defer b.lock.Unlock()
 
 	data := make(map[statePermutation]int)
+
+	for token := range b.counters {
+		// 'idle' state will ensure the metric is always present, even if no
+		// builds are being processed at the moment
+		idleState := statePermutation{
+			runner:        helpers.ShortenToken(token),
+			buildState:    "idle",
+			buildStage:    "idle",
+			executorStage: "idle",
+		}
+		data[idleState] = 0
+	}
+
 	for _, build := range b.builds {
 		state := newStatePermutationFromBuild(build)
 		data[state]++
 	}
+
 	return data
 }
 
@@ -291,27 +304,18 @@ func (b *buildsHelper) ListJobsHandler(w http.ResponseWriter, r *http.Request) {
 	w.WriteHeader(http.StatusOK)
 
 	for _, job := range b.builds {
-		url := CreateJobURL(job.RepoCleanURL(), job.ID)
-
 		_, _ = fmt.Fprintf(
 			w,
 			"url=%s state=%s stage=%s executor_stage=%s duration=%s\n",
-			url,
-			job.CurrentState,
-			job.CurrentStage,
+			job.JobURL(),
+			job.CurrentState(),
+			job.CurrentStage(),
 			job.CurrentExecutorStage(),
 			job.Duration(),
 		)
 	}
 }
 
-func CreateJobURL(projectURL string, jobID int) string {
-	r := regexp.MustCompile(`(\.git$)?`)
-	URL := r.ReplaceAllString(projectURL, "")
-
-	return fmt.Sprintf("%s/-/jobs/%d", URL, jobID)
-}
-
 func newBuildsHelper() buildsHelper {
 	return buildsHelper{
 		jobsTotal: prometheus.NewCounterVec(
