@@ -1,24 +1,29 @@
 package helpers
 
 import (
+	"context"
 	"io"
 	"io/ioutil"
 	"net/http"
 	"os"
 	"path/filepath"
+	"strconv"
 	"time"
 
 	"github.com/sirupsen/logrus"
 	"github.com/urfave/cli"
 
+	"gitlab.com/gitlab-org/gitlab-runner/commands/helpers/archive"
+	"gitlab.com/gitlab-org/gitlab-runner/commands/helpers/meter"
 	"gitlab.com/gitlab-org/gitlab-runner/common"
-	"gitlab.com/gitlab-org/gitlab-runner/helpers/archives"
 	url_helpers "gitlab.com/gitlab-org/gitlab-runner/helpers/url"
 	"gitlab.com/gitlab-org/gitlab-runner/log"
 )
 
 type CacheExtractorCommand struct {
 	retryHelper
+	meter.TransferMeterCommand
+
 	File    string `long:"file" description:"The file containing your cache artifacts"`
 	URL     string `long:"url" description:"URL of remote cache resource"`
 	Timeout int    `long:"timeout" description:"Overall timeout for cache downloading request (in minutes)"`
@@ -40,6 +45,15 @@ func checkIfUpToDate(path string, resp *http.Response) (bool, time.Time) {
 	return fi != nil && !date.After(fi.ModTime()), date
 }
 
+func getRemoteCacheSize(resp *http.Response) int64 {
+	length, _ := strconv.Atoi(resp.Header.Get("Content-Length"))
+	if length <= 0 {
+		return meter.UnknownTotalSize
+	}
+
+	return int64(length)
+}
+
 func (c *CacheExtractorCommand) download(_ int) error {
 	err := os.MkdirAll(filepath.Dir(c.File), 0700)
 	if err != nil {
@@ -63,22 +77,34 @@ func (c *CacheExtractorCommand) download(_ int) error {
 	if err != nil {
 		return err
 	}
+
 	defer func() {
 		_ = file.Close()
 		_ = os.Remove(file.Name())
 	}()
 
 	logrus.Infoln("Downloading", filepath.Base(c.File), "from", url_helpers.CleanURL(c.URL))
-	_, err = io.Copy(file, resp.Body)
+
+	writer := meter.NewWriter(
+		file,
+		c.TransferMeterFrequency,
+		meter.LabelledRateFormat(os.Stdout, "Downloading cache", getRemoteCacheSize(resp)),
+	)
+
+	// Close() is checked properly bellow, where the file handling is being finalized
+	defer func() { _ = writer.Close() }()
+
+	_, err = io.Copy(writer, resp.Body)
 	if err != nil {
 		return retryableErr{err: err}
 	}
+
 	err = os.Chtimes(file.Name(), time.Now(), date)
 	if err != nil {
 		return err
 	}
 
-	err = file.Close()
+	err = writer.Close()
 	if err != nil {
 		return err
 	}
@@ -105,17 +131,22 @@ func (c *CacheExtractorCommand) getCache() (*http.Response, error) {
 	return resp, retryOnServerError(resp)
 }
 
-func (c *CacheExtractorCommand) Execute(context *cli.Context) {
+func (c *CacheExtractorCommand) Execute(cliContext *cli.Context) {
 	log.SetRunnerFormatter()
 
+	wd, err := os.Getwd()
+	if err != nil {
+		logrus.Fatalln("Unable to get working directory")
+	}
+
 	if c.File == "" {
-		logrus.Fatalln("Missing cache file")
+		warningln("Missing cache file")
 	}
 
 	if c.URL != "" {
 		err := c.doRetry(c.download)
 		if err != nil {
-			logrus.Fatalln(err)
+			warningln(err)
 		}
 	} else {
 		logrus.Infoln(
@@ -123,10 +154,29 @@ func (c *CacheExtractorCommand) Execute(context *cli.Context) {
 				"Instead a local version of cache will be extracted.")
 	}
 
-	err := archives.ExtractZipFile(c.File)
-	if err != nil && !os.IsNotExist(err) {
+	f, size, err := openZip(c.File)
+	if os.IsNotExist(err) {
+		return
+	}
+	if err != nil {
 		logrus.Fatalln(err)
 	}
+	defer f.Close()
+
+	extractor, err := archive.NewExtractor(archive.Zip, f, size, wd)
+	if err != nil {
+		logrus.Fatalln(err)
+	}
+
+	err = extractor.Extract(context.Background())
+	if err != nil {
+		logrus.Fatalln(err)
+	}
+}
+
+func warningln(args interface{}) {
+	logrus.Warningln(args)
+	logrus.Exit(1)
 }
 
 func init() {
