@@ -5,13 +5,13 @@ import (
 	"os"
 	"runtime"
 
-	"github.com/ayufan/golang-kardianos-service"
+	"github.com/kardianos/service"
 	"github.com/sirupsen/logrus"
 	"github.com/urfave/cli"
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers"
-	"gitlab.com/gitlab-org/gitlab-runner/helpers/service"
+	service_helpers "gitlab.com/gitlab-org/gitlab-runner/helpers/service"
 )
 
 const (
@@ -54,18 +54,31 @@ func runServiceInstall(s service.Service, c *cli.Context) error {
 	return service.Control(s, "install")
 }
 
-func runServiceStatus(displayName string, s service.Service) error {
-	err := s.Status()
-	if err == nil {
-		fmt.Println(displayName+":", "Service is running!")
-	} else {
-		fmt.Fprintln(os.Stderr, displayName+":", err)
+func runServiceStatus(displayName string, s service.Service) {
+	status, err := s.Status()
+
+	description := ""
+	switch status {
+	case service.StatusRunning:
+		description = "Service is running"
+	case service.StatusStopped:
+		description = "Service has stopped"
+	default:
+		description = "Service status unknown"
+		if err != nil {
+			description = err.Error()
+		}
+	}
+
+	if status != service.StatusRunning {
+		fmt.Fprintf(os.Stderr, "%s: %s\n", displayName, description)
 		os.Exit(1)
 	}
-	return nil
+
+	fmt.Printf("%s: %s\n", displayName, description)
 }
 
-func getServiceArguments(c *cli.Context) (arguments []string) {
+func GetServiceArguments(c *cli.Context) (arguments []string) {
 	if wd := c.String("working-directory"); wd != "" {
 		arguments = append(arguments, "--working-directory", wd)
 	}
@@ -78,7 +91,12 @@ func getServiceArguments(c *cli.Context) (arguments []string) {
 		arguments = append(arguments, "--service", sn)
 	}
 
+	// syslogging doesn't make sense for systemd systems as those log straight to journald
 	syslog := !c.IsSet("syslog") || c.Bool("syslog")
+	if service.Platform() == "linux-systemd" && !c.IsSet("syslog") {
+		syslog = false
+	}
+
 	if syslog {
 		arguments = append(arguments, "--syslog")
 	}
@@ -86,46 +104,18 @@ func getServiceArguments(c *cli.Context) (arguments []string) {
 	return
 }
 
-func createServiceConfig(c *cli.Context) (svcConfig *service.Config) {
-	svcConfig = &service.Config{
+func createServiceConfig(c *cli.Context) *service.Config {
+	config := &service.Config{
 		Name:        c.String("service"),
 		DisplayName: c.String("service"),
 		Description: defaultDescription,
-		Arguments:   []string{"run"},
+		Arguments:   append([]string{"run"}, GetServiceArguments(c)...),
 	}
-	svcConfig.Arguments = append(svcConfig.Arguments, getServiceArguments(c)...)
 
-	switch runtime.GOOS {
-	case "linux":
-		if os.Getuid() != 0 {
-			logrus.Fatal("Please run the commands as root")
-		}
-		if user := c.String("user"); user != "" {
-			svcConfig.Arguments = append(svcConfig.Arguments, "--user", user)
-		}
+	// setup os specific service config
+	setupOSServiceConfig(c, config)
 
-	case "darwin":
-		svcConfig.Option = service.KeyValue{
-			"KeepAlive":   true,
-			"RunAtLoad":   true,
-			"UserService": os.Getuid() != 0,
-		}
-
-		if user := c.String("user"); user != "" {
-			if os.Getuid() == 0 {
-				svcConfig.Arguments = append(svcConfig.Arguments, "--user", user)
-			} else {
-				logrus.Fatalln("The --user is not supported for non-root users")
-			}
-		}
-
-	case "windows":
-		svcConfig.Option = service.KeyValue{
-			"Password": c.String("password"),
-		}
-		svcConfig.UserName = c.String("user")
-	}
-	return
+	return config
 }
 
 func RunServiceControl(c *cli.Context) {
@@ -140,7 +130,7 @@ func RunServiceControl(c *cli.Context) {
 	case "install":
 		err = runServiceInstall(s, c)
 	case "status":
-		err = runServiceStatus(svcConfig.DisplayName, s)
+		runServiceStatus(svcConfig.DisplayName, s)
 	default:
 		err = service.Control(s, c.Command.Name)
 	}
@@ -150,7 +140,7 @@ func RunServiceControl(c *cli.Context) {
 	}
 }
 
-func getFlags() []cli.Flag {
+func GetFlags() []cli.Flag {
 	return []cli.Flag{
 		cli.StringFlag{
 			Name:  "service, n",
@@ -160,34 +150,39 @@ func getFlags() []cli.Flag {
 	}
 }
 
-func getInstallFlags() []cli.Flag {
-	installFlags := getFlags()
-	installFlags = append(installFlags, cli.StringFlag{
-		Name:  "working-directory, d",
-		Value: helpers.GetCurrentWorkingDirectory(),
-		Usage: "Specify custom root directory where all data are stored",
-	})
-	installFlags = append(installFlags, cli.StringFlag{
-		Name:  "config, c",
-		Value: getDefaultConfigFile(),
-		Usage: "Specify custom config file",
-	})
-	installFlags = append(installFlags, cli.BoolFlag{
-		Name:  "syslog",
-		Usage: "Setup system logging integration",
-	})
+func GetInstallFlags() []cli.Flag {
+	installFlags := GetFlags()
+	installFlags = append(
+		installFlags,
+		cli.StringFlag{
+			Name:  "working-directory, d",
+			Value: helpers.GetCurrentWorkingDirectory(),
+			Usage: "Specify custom root directory where all data are stored",
+		},
+		cli.StringFlag{
+			Name:  "config, c",
+			Value: GetDefaultConfigFile(),
+			Usage: "Specify custom config file",
+		},
+		cli.BoolFlag{
+			Name:  "syslog",
+			Usage: "Setup system logging integration",
+		},
+	)
 
-	if runtime.GOOS == "windows" {
-		installFlags = append(installFlags, cli.StringFlag{
-			Name:  "user, u",
-			Value: "",
-			Usage: "Specify user-name to secure the runner",
-		})
-		installFlags = append(installFlags, cli.StringFlag{
-			Name:  "password, p",
-			Value: "",
-			Usage: "Specify user password to install service (required)",
-		})
+	if runtime.GOOS == osTypeWindows {
+		installFlags = append(
+			installFlags,
+			cli.StringFlag{
+				Name:  "user, u",
+				Value: "",
+				Usage: "Specify user-name to secure the runner",
+			},
+			cli.StringFlag{
+				Name:  "password, p",
+				Value: "",
+				Usage: "Specify user password to install service (required)",
+			})
 	} else if os.Getuid() == 0 {
 		installFlags = append(installFlags, cli.StringFlag{
 			Name:  "user, u",
@@ -200,8 +195,8 @@ func getInstallFlags() []cli.Flag {
 }
 
 func init() {
-	flags := getFlags()
-	installFlags := getInstallFlags()
+	flags := GetFlags()
+	installFlags := GetInstallFlags()
 
 	common.RegisterCommand(cli.Command{
 		Name:   "install",
