@@ -1,6 +1,7 @@
 package commands
 
 import (
+	"context"
 	"os"
 	"os/signal"
 	"syscall"
@@ -73,7 +74,7 @@ func (r *RunSingleCommand) postBuild() {
 }
 
 func (r *RunSingleCommand) processBuild(data common.ExecutorData, abortSignal chan os.Signal) error {
-	jobData, healthy := r.network.RequestJob(r.RunnerConfig, nil)
+	jobData, healthy := r.network.RequestJob(context.Background(), r.RunnerConfig, nil)
 	if !healthy {
 		logrus.Println("Runner is not healthy!")
 		select {
