@@ -1,3 +1,6 @@
+//go:build !integration
+// +build !integration
+
 package common
 
 import (
@@ -173,15 +176,15 @@ func TestJobImageExposed(t *testing.T) {
 		expectImageName string
 	}{
 		"normal image exposed": {
-			image:           "alpine:3.11",
+			image:           "alpine:3.14",
 			expectVarExists: true,
-			expectImageName: "alpine:3.11",
+			expectImageName: "alpine:3.14",
 		},
 		"image with variable expansion": {
-			image:           "${IMAGE}:3.11",
+			image:           "${IMAGE}:3.14",
 			vars:            []JobVariable{{Key: "IMAGE", Value: "alpine", Public: true}},
 			expectVarExists: true,
-			expectImageName: "alpine:3.11",
+			expectImageName: "alpine:3.14",
 		},
 		"no image specified": {
 			image:           "",
@@ -309,7 +312,7 @@ func TestPrepareFailureOnBuildError(t *testing.T) {
 	err := build.Run(&Config{}, &Trace{Writer: os.Stdout})
 
 	expectedErr := new(BuildError)
-	assert.True(t, errors.Is(err, expectedErr), "expected: %#v, got: %#v", expectedErr, err)
+	assert.ErrorIs(t, err, expectedErr)
 }
 
 func TestPrepareEnvironmentFailure(t *testing.T) {
@@ -346,7 +349,7 @@ func TestPrepareEnvironmentFailure(t *testing.T) {
 	}
 
 	err = build.Run(&Config{}, &Trace{Writer: os.Stdout})
-	assert.True(t, errors.Is(err, testErr))
+	assert.ErrorIs(t, err, testErr)
 }
 
 func TestJobFailure(t *testing.T) {
@@ -358,10 +361,11 @@ func TestJobFailure(t *testing.T) {
 	executor.On("Cleanup").Once()
 
 	// Set up a failing a build script
-	thrownErr := &BuildError{Inner: errors.New("test error")}
+	thrownErr := &BuildError{Inner: errors.New("test error"), ExitCode: 1}
 	executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"})
 	executor.On("Run", matchBuildStage(BuildStagePrepare)).Return(nil).Once()
-	executor.On("Run", mock.Anything).Return(thrownErr).Times(2)
+	executor.On("Run", mock.Anything).Return(thrownErr).Times(3)
+	executor.On("Run", matchBuildStage(BuildStageCleanup)).Return(nil).Once()
 	executor.On("Finish", thrownErr).Once()
 
 	RegisterExecutorProvider("build-run-job-failure", provider)
@@ -382,13 +386,14 @@ func TestJobFailure(t *testing.T) {
 	trace.On("Write", mock.Anything).Return(0, nil)
 	trace.On("IsStdout").Return(true)
 	trace.On("SetCancelFunc", mock.Anything).Once()
+	trace.On("SetAbortFunc", mock.Anything).Once()
 	trace.On("SetMasked", mock.Anything).Once()
-	trace.On("Fail", thrownErr, ScriptFailure).Once()
+	trace.On("Fail", thrownErr, JobFailureData{Reason: ScriptFailure, ExitCode: 1}).Once()
 
 	err = build.Run(&Config{}, trace)
 
 	expectedErr := new(BuildError)
-	assert.True(t, errors.Is(err, expectedErr), "expected: %#v, got: %#v", expectedErr, err)
+	assert.ErrorIs(t, err, expectedErr)
 }
 
 func TestJobFailureOnExecutionTimeout(t *testing.T) {
@@ -416,15 +421,16 @@ func TestJobFailureOnExecutionTimeout(t *testing.T) {
 	trace.On("Write", mock.Anything).Return(0, nil)
 	trace.On("IsStdout").Return(true)
 	trace.On("SetCancelFunc", mock.Anything).Once()
+	trace.On("SetAbortFunc", mock.Anything).Once()
 	trace.On("SetMasked", mock.Anything).Once()
-	trace.On("Fail", mock.Anything, JobExecutionTimeout).Run(func(arguments mock.Arguments) {
+	trace.On("Fail", mock.Anything, JobFailureData{Reason: JobExecutionTimeout}).Run(func(arguments mock.Arguments) {
 		assert.Error(t, arguments.Get(0).(error))
 	}).Once()
 
 	err := build.Run(&Config{}, trace)
 
 	expectedErr := &BuildError{FailureReason: JobExecutionTimeout}
-	assert.True(t, errors.Is(err, expectedErr), "expected: %#v, got: %#v", expectedErr, err)
+	assert.ErrorIs(t, err, expectedErr)
 }
 
 func TestRunFailureRunsAfterScriptAndArtifactsOnFailure(t *testing.T) {
@@ -444,6 +450,8 @@ func TestRunFailureRunsAfterScriptAndArtifactsOnFailure(t *testing.T) {
 	executor.On("Run", matchBuildStage("step_script")).Return(errors.New("build fail")).Once()
 	executor.On("Run", matchBuildStage(BuildStageAfterScript)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageUploadOnFailureArtifacts)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageArchiveOnFailureCache)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageCleanup)).Return(nil).Once()
 	executor.On("Finish", errors.New("build fail")).Once()
 
 	RegisterExecutorProvider("build-run-run-failure", provider)
@@ -474,7 +482,9 @@ func TestGetSourcesRunFailure(t *testing.T) {
 	executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"})
 	executor.On("Run", matchBuildStage(BuildStagePrepare)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageGetSources)).Return(errors.New("build fail")).Times(3)
+	executor.On("Run", matchBuildStage(BuildStageArchiveOnFailureCache)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageUploadOnFailureArtifacts)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageCleanup)).Return(nil).Once()
 	executor.On("Finish", errors.New("build fail")).Once()
 
 	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
@@ -497,7 +507,9 @@ func TestArtifactDownloadRunFailure(t *testing.T) {
 	executor.On("Run", matchBuildStage(BuildStageGetSources)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageRestoreCache)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageDownloadArtifacts)).Return(errors.New("build fail")).Times(3)
+	executor.On("Run", matchBuildStage(BuildStageArchiveOnFailureCache)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageUploadOnFailureArtifacts)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageCleanup)).Return(nil).Once()
 	executor.On("Finish", errors.New("build fail")).Once()
 
 	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
@@ -515,15 +527,16 @@ func TestArtifactUploadRunFailure(t *testing.T) {
 	executor.On("Cleanup").Once()
 
 	// Successful build script
-	executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"}).Times(8)
+	executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"}).Times(9)
 	executor.On("Run", matchBuildStage(BuildStagePrepare)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageGetSources)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageRestoreCache)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageDownloadArtifacts)).Return(nil).Once()
 	executor.On("Run", matchBuildStage("step_script")).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageAfterScript)).Return(nil).Once()
-	executor.On("Run", matchBuildStage(BuildStageArchiveCache)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageArchiveOnSuccessCache)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageUploadOnSuccessArtifacts)).Return(errors.New("upload fail")).Once()
+	executor.On("Run", matchBuildStage(BuildStageCleanup)).Return(nil).Once()
 	executor.On("Finish", errors.New("upload fail")).Once()
 
 	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
@@ -539,6 +552,58 @@ func TestArtifactUploadRunFailure(t *testing.T) {
 	assert.EqualError(t, err, "upload fail")
 }
 
+func TestArchiveCacheOnScriptFailure(t *testing.T) {
+	executor, provider := setupMockExecutorAndProvider()
+	defer executor.AssertExpectations(t)
+	defer provider.AssertExpectations(t)
+	executor.On("Prepare", mock.Anything, mock.Anything, mock.Anything).
+		Return(nil).Once()
+	executor.On("Cleanup").Once()
+
+	// Fail a build script
+	executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"}).Times(9)
+	executor.On("Run", matchBuildStage(BuildStagePrepare)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageGetSources)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageRestoreCache)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageDownloadArtifacts)).Return(nil).Once()
+	executor.On("Run", matchBuildStage("step_script")).Return(errors.New("script failure")).Once()
+	executor.On("Run", matchBuildStage(BuildStageAfterScript)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageArchiveOnFailureCache)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageUploadOnFailureArtifacts)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageCleanup)).Return(nil).Once()
+	executor.On("Finish", errors.New("script failure")).Once()
+
+	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
+	err := build.Run(&Config{}, &Trace{Writer: os.Stdout})
+	assert.EqualError(t, err, "script failure")
+}
+
+func TestUploadArtifactsOnArchiveCacheFailure(t *testing.T) {
+	executor, provider := setupMockExecutorAndProvider()
+	defer executor.AssertExpectations(t)
+	defer provider.AssertExpectations(t)
+	executor.On("Prepare", mock.Anything, mock.Anything, mock.Anything).
+		Return(nil).Once()
+	executor.On("Cleanup").Once()
+
+	// Successful build script
+	executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"}).Times(9)
+	executor.On("Run", matchBuildStage(BuildStagePrepare)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageGetSources)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageRestoreCache)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageDownloadArtifacts)).Return(nil).Once()
+	executor.On("Run", matchBuildStage("step_script")).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageAfterScript)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageArchiveOnSuccessCache)).Return(errors.New("cache failure")).Once()
+	executor.On("Run", matchBuildStage(BuildStageUploadOnSuccessArtifacts)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageCleanup)).Return(nil).Once()
+	executor.On("Finish", errors.New("cache failure")).Once()
+
+	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
+	err := build.Run(&Config{}, &Trace{Writer: os.Stdout})
+	assert.EqualError(t, err, "cache failure")
+}
+
 func TestRestoreCacheRunFailure(t *testing.T) {
 	executor, provider := setupMockExecutorAndProvider()
 	defer executor.AssertExpectations(t)
@@ -552,7 +617,9 @@ func TestRestoreCacheRunFailure(t *testing.T) {
 	executor.On("Run", matchBuildStage(BuildStagePrepare)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageGetSources)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageRestoreCache)).Return(errors.New("build fail")).Times(3)
+	executor.On("Run", matchBuildStage(BuildStageArchiveOnFailureCache)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageUploadOnFailureArtifacts)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageCleanup)).Return(nil).Once()
 	executor.On("Finish", errors.New("build fail")).Once()
 
 	build := registerExecutorWithSuccessfulBuild(t, provider, new(RunnerConfig))
@@ -674,15 +741,48 @@ func TestDebugTrace(t *testing.T) {
 }
 
 func TestDefaultEnvVariables(t *testing.T) {
-	buildDir := "/tmp/test-build/dir"
-	build := Build{
-		BuildDir: buildDir,
+	tests := map[string]struct {
+		buildDir      string
+		expectedValue string
+	}{
+		"UNIX-style BuildDir": {
+			buildDir:      "/tmp/test-build/dir",
+			expectedValue: "CI_PROJECT_DIR=/tmp/test-build/dir",
+		},
+		// The next four tests' expected value will depend on the platform running the tests
+		"Windows UNC-style BuildDir (extended-length path support)": {
+			buildDir:      `\\?\C:\tmp\test-build\dir`,
+			expectedValue: "CI_PROJECT_DIR=" + filepath.FromSlash("//?/C:/tmp/test-build/dir"),
+		},
+		"Windows UNC-style BuildDir": {
+			buildDir:      `\\host\share\tmp\test-build\dir`,
+			expectedValue: "CI_PROJECT_DIR=" + filepath.FromSlash("//host/share/tmp/test-build/dir"),
+		},
+		"Windows-style BuildDir (CMD or PS)": {
+			buildDir:      `C:\tmp\test-build\dir`,
+			expectedValue: "CI_PROJECT_DIR=" + filepath.FromSlash("C:/tmp/test-build/dir"),
+		},
+		"Windows-style BuildDir with forward slashes and drive letter": {
+			buildDir:      "C:/tmp/test-build/dir",
+			expectedValue: "CI_PROJECT_DIR=" + filepath.FromSlash("C:/tmp/test-build/dir"),
+		},
+		"Windows-style BuildDir in MSYS bash executor and drive letter)": {
+			buildDir:      "/c/tmp/test-build/dir",
+			expectedValue: "CI_PROJECT_DIR=/c/tmp/test-build/dir",
+		},
 	}
 
-	vars := build.GetAllVariables().StringList()
+	for name, test := range tests {
+		t.Run(name, func(t *testing.T) {
+			build := new(Build)
+			build.BuildDir = test.buildDir
 
-	assert.Contains(t, vars, "CI_PROJECT_DIR="+filepath.FromSlash(buildDir))
-	assert.Contains(t, vars, "CI_SERVER=yes")
+			vars := build.GetAllVariables().StringList()
+
+			assert.Contains(t, vars, test.expectedValue)
+			assert.Contains(t, vars, "CI_SERVER=yes")
+		})
+	}
 }
 
 func TestSharedEnvVariables(t *testing.T) {
@@ -712,116 +812,136 @@ func TestSharedEnvVariables(t *testing.T) {
 }
 
 func TestGetRemoteURL(t *testing.T) {
-	testCases := []struct {
-		runner RunnerSettings
-		result string
+	const (
+		exampleJobToken    = "job-token"
+		exampleRepoURL     = "http://gitlab-ci-token:job-token@test.remote/my/project.git"
+		exampleProjectPath = "my/project"
+	)
+
+	testCases := map[string]struct {
+		runner                    RunnerSettings
+		jobTokenVariableOverwrite string
+		expectedURL               string
 	}{
-		{
+		"using clone_url with http protocol": {
 			runner: RunnerSettings{
 				CloneURL: "http://test.local/",
 			},
-			result: "http://gitlab-ci-token:1234567@test.local/h5bp/html5-boilerplate.git",
+			expectedURL: "http://gitlab-ci-token:job-token@test.local/my/project.git",
 		},
-		{
+		"using clone_url with https protocol": {
 			runner: RunnerSettings{
 				CloneURL: "https://test.local",
 			},
-			result: "https://gitlab-ci-token:1234567@test.local/h5bp/html5-boilerplate.git",
+			expectedURL: "https://gitlab-ci-token:job-token@test.local/my/project.git",
 		},
-		{
-			runner: RunnerSettings{},
-			result: "http://fallback.url",
+		"not using clone_url": {
+			runner:      RunnerSettings{},
+			expectedURL: exampleRepoURL,
+		},
+		"overwriting job token with variable and clone_url": {
+			runner: RunnerSettings{
+				CloneURL: "https://test.local",
+			},
+			jobTokenVariableOverwrite: "wrong-token",
+			expectedURL:               "https://gitlab-ci-token:job-token@test.local/my/project.git",
+		},
+		"overwriting job token with variable and no clone_url": {
+			runner:                    RunnerSettings{},
+			jobTokenVariableOverwrite: "wrong-token",
+			expectedURL:               exampleRepoURL,
 		},
 	}
 
-	for _, tc := range testCases {
-		build := &Build{
-			Runner: &RunnerConfig{
-				RunnerSettings: tc.runner,
-			},
-			allVariables: JobVariables{
-				JobVariable{Key: "CI_JOB_TOKEN", Value: "1234567"},
-				JobVariable{Key: "CI_PROJECT_PATH", Value: "h5bp/html5-boilerplate"},
-			},
-			JobResponse: JobResponse{
-				GitInfo: GitInfo{RepoURL: "http://fallback.url"},
-			},
-		}
+	for tn, tc := range testCases {
+		t.Run(tn, func(t *testing.T) {
+			build := &Build{
+				Runner: &RunnerConfig{
+					RunnerSettings: tc.runner,
+				},
+				allVariables: JobVariables{
+					JobVariable{Key: "CI_PROJECT_PATH", Value: exampleProjectPath},
+				},
+				JobResponse: JobResponse{
+					Token: exampleJobToken,
+					GitInfo: GitInfo{
+						RepoURL: exampleRepoURL,
+					},
+				},
+			}
 
-		assert.Equal(t, tc.result, build.GetRemoteURL())
+			if tc.jobTokenVariableOverwrite != "" {
+				build.allVariables = append(build.allVariables, JobVariable{
+					Key:   "CI_JOB_TOKEN",
+					Value: tc.jobTokenVariableOverwrite,
+				})
+			}
+
+			assert.Equal(t, tc.expectedURL, build.GetRemoteURL())
+		})
 	}
 }
 
-type featureFlagOnTestCase struct {
-	value          string
-	expectedStatus bool
-	expectedError  bool
-}
-
 func TestIsFeatureFlagOn(t *testing.T) {
-	hook := test.NewGlobal()
+	const testFF = "FF_TEST_FEATURE"
 
-	tests := map[string]featureFlagOnTestCase{
+	tests := map[string]struct {
+		featureFlagCfg map[string]bool
+		value          string
+		expectedStatus bool
+	}{
 		"no value": {
 			value:          "",
 			expectedStatus: false,
-			expectedError:  false,
 		},
 		"true": {
 			value:          "true",
 			expectedStatus: true,
-			expectedError:  false,
 		},
 		"1": {
 			value:          "1",
 			expectedStatus: true,
-			expectedError:  false,
 		},
 		"false": {
 			value:          "false",
 			expectedStatus: false,
-			expectedError:  false,
 		},
 		"0": {
 			value:          "0",
 			expectedStatus: false,
-			expectedError:  false,
 		},
 		"invalid value": {
 			value:          "test",
 			expectedStatus: false,
-			expectedError:  true,
+		},
+		"feature flag set inside config.toml take precedence": {
+			featureFlagCfg: map[string]bool{
+				testFF: true,
+			},
+			value:          "false",
+			expectedStatus: true,
 		},
 	}
 
 	for name, testCase := range tests {
 		t.Run(name, func(t *testing.T) {
 			build := new(Build)
+			build.Runner = &RunnerConfig{
+				RunnerSettings: RunnerSettings{
+					FeatureFlags: testCase.featureFlagCfg,
+				},
+			}
 			build.Variables = JobVariables{
-				{Key: "FF_TEST_FEATURE", Value: testCase.value},
+				{Key: testFF, Value: testCase.value},
 			}
 
-			status := build.IsFeatureFlagOn("FF_TEST_FEATURE")
+			status := build.IsFeatureFlagOn(testFF)
 			assert.Equal(t, testCase.expectedStatus, status)
-
-			entry := hook.LastEntry()
-			if testCase.expectedError {
-				require.NotNil(t, entry)
-
-				logrusOutput, err := entry.String()
-				require.NoError(t, err)
-
-				assert.Contains(t, logrusOutput, "Error while parsing the value of feature flag")
-			} else {
-				assert.Nil(t, entry)
-			}
-
-			hook.Reset()
 		})
 	}
 }
 
-func TestAllowToOverwriteFeatureFlagWithRunnerVariables(t *testing.T) {
+func TestIsFeatureFlagOn_SetWithRunnerVariables(t *testing.T) {
 	tests := map[string]struct {
 		variable      string
 		expectedValue bool
@@ -855,6 +975,61 @@ func TestAllowToOverwriteFeatureFlagWithRunnerVariables(t *testing.T) {
 	}
 }
 
+func TestIsFeatureFlagOn_Precedence(t *testing.T) {
+	const testFF = "FF_TEST_FEATURE"
+
+	t.Run("config takes precedence over job variable", func(t *testing.T) {
+		b := &Build{
+			Runner: &RunnerConfig{
+				RunnerSettings: RunnerSettings{
+					FeatureFlags: map[string]bool{
+						testFF: true,
+					},
+				},
+			},
+			JobResponse: JobResponse{
+				Variables: JobVariables{
+					{Key: testFF, Value: "false"},
+				},
+			},
+		}
+
+		assert.True(t, b.IsFeatureFlagOn(testFF))
+	})
+
+	t.Run("config takes precedence over configured environments", func(t *testing.T) {
+		b := &Build{
+			Runner: &RunnerConfig{
+				RunnerSettings: RunnerSettings{
+					FeatureFlags: map[string]bool{
+						testFF: true,
+					},
+					Environment: []string{testFF + "=false"},
+				},
+			},
+		}
+
+		assert.True(t, b.IsFeatureFlagOn(testFF))
+	})
+
+	t.Run("variable defined at job take precedence over configured environments", func(t *testing.T) {
+		b := &Build{
+			Runner: &RunnerConfig{
+				RunnerSettings: RunnerSettings{
+					Environment: []string{testFF + "=false"},
+				},
+			},
+			JobResponse: JobResponse{
+				Variables: JobVariables{
+					{Key: testFF, Value: "true"},
+				},
+			},
+		}
+
+		assert.True(t, b.IsFeatureFlagOn(testFF))
+	})
+}
+
 func TestStartBuild(t *testing.T) {
 	type startBuildArgs struct {
 		rootDir               string
@@ -926,6 +1101,22 @@ func TestStartBuild(t *testing.T) {
 			expectedCacheDir: "/cache/test-namespace/test-repo",
 			expectedError:    false,
 		},
+		"out-of-bounds GIT_CLONE_PATH was specified": {
+			args: startBuildArgs{
+				rootDir:               "/builds",
+				cacheDir:              "/cache",
+				customBuildDirEnabled: true,
+				sharedDir:             false,
+			},
+			jobVariables: JobVariables{
+				{
+					Key:    "GIT_CLONE_PATH",
+					Value:  "/builds/../outside",
+					Public: true,
+				},
+			},
+			expectedError: true,
+		},
 		"custom build disabled": {
 			args: startBuildArgs{
 				rootDir:               "/builds",
@@ -995,9 +1186,11 @@ func TestSkipBuildStageFeatureFlag(t *testing.T) {
 		"false",
 	}
 
-	s := MockShell{}
+	s := new(MockShell)
+	defer s.AssertExpectations(t)
+
 	s.On("GetName").Return("skip-build-stage-shell")
-	RegisterShell(&s)
+	RegisterShell(s)
 
 	for _, value := range featureFlagValues {
 		t.Run(value, func(t *testing.T) {
@@ -1097,7 +1290,7 @@ func TestWaitForTerminal(t *testing.T) {
 			require.NoError(t, err)
 			build.Session = sess
 
-			srv := httptest.NewServer(build.Session.Mux())
+			srv := httptest.NewServer(build.Session.Handler())
 			defer srv.Close()
 
 			mockConn := terminal.MockConn{}
@@ -1286,6 +1479,42 @@ func TestGitFetchFlags(t *testing.T) {
 	}
 }
 
+func TestGitSubmoduleUpdateFlags(t *testing.T) {
+	tests := map[string]struct {
+		value          string
+		expectedResult []string
+	}{
+		"empty update flags": {
+			value:          "",
+			expectedResult: []string{},
+		},
+		"use custom update flags": {
+			value:          "custom-flags",
+			expectedResult: []string{"custom-flags"},
+		},
+		"use custom update flags with multiple arguments": {
+			value:          "--remote --jobs 4",
+			expectedResult: []string{"--remote", "--jobs", "4"},
+		},
+	}
+
+	for name, test := range tests {
+		t.Run(name, func(t *testing.T) {
+			build := &Build{
+				Runner: &RunnerConfig{},
+				JobResponse: JobResponse{
+					Variables: JobVariables{
+						{Key: "GIT_SUBMODULE_UPDATE_FLAGS", Value: test.value},
+					},
+				},
+			}
+
+			result := build.GetGitSubmoduleUpdateFlags()
+			assert.Equal(t, test.expectedResult, result)
+		})
+	}
+}
+
 func TestDefaultVariables(t *testing.T) {
 	tests := map[string]struct {
 		jobVariables  JobVariables
@@ -1375,11 +1604,11 @@ func TestBuildFinishTimeout(t *testing.T) {
 
 func TestProjectUniqueName(t *testing.T) {
 	tests := map[string]struct {
-		build        Build
+		build        *Build
 		expectedName string
 	}{
 		"project non rfc1132 unique name": {
-			build: Build{
+			build: &Build{
 				Runner: &RunnerConfig{
 					RunnerCredentials: RunnerCredentials{
 						Token: "Ze_n8E6en622WxxSg4r8",
@@ -1394,24 +1623,8 @@ func TestProjectUniqueName(t *testing.T) {
 			},
 			expectedName: "runner-zen8e6e-project-1234567890-concurrent-0",
 		},
-		"project non rfc1132 unique name longer than 63 char": {
-			build: Build{
-				Runner: &RunnerConfig{
-					RunnerCredentials: RunnerCredentials{
-						Token: "Ze_n8E6en622WxxSg4r8",
-					},
-				},
-				JobResponse: JobResponse{
-					JobInfo: JobInfo{
-						ProjectID: 123456789012345,
-					},
-				},
-				ProjectRunnerID: 123456789012345,
-			},
-			expectedName: "runner-zen8e6e-project-123456789012345-concurrent-1234567890123",
-		},
 		"project normal unique name": {
-			build: Build{
+			build: &Build{
 				Runner: &RunnerConfig{
 					RunnerCredentials: RunnerCredentials{
 						Token: "xYzWabc-Ij3xlKjmoPO9",
@@ -1506,12 +1719,52 @@ func TestBuild_GetExecutorJobSectionAttempts(t *testing.T) {
 			}
 
 			attempts, err := build.GetExecutorJobSectionAttempts()
-			assert.True(t, errors.Is(err, tt.expectedErr))
+			assert.ErrorIs(t, err, tt.expectedErr)
 			assert.Equal(t, tt.expectedAttempts, attempts)
 		})
 	}
 }
 
+func TestBuild_getFeatureFlagInfo(t *testing.T) {
+	const changedFeatureFlags = "FF_CMD_DISABLE_DELAYED_ERROR_LEVEL_EXPANSION:true"
+	tests := []struct {
+		value          string
+		expectedStatus string
+	}{
+		{
+			value:          "true",
+			expectedStatus: changedFeatureFlags,
+		},
+		{
+			value:          "1",
+			expectedStatus: changedFeatureFlags,
+		},
+		{
+			value:          "invalid",
+			expectedStatus: "",
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.value, func(t *testing.T) {
+			b := Build{
+				JobResponse: JobResponse{
+					Variables: JobVariables{
+						{
+							Key:    featureflags.CmdDisableDelayedErrorLevelExpansion,
+							Value:  tt.value,
+							Public: true,
+						},
+					},
+				},
+				Runner: &RunnerConfig{},
+			}
+
+			assert.Equal(t, tt.expectedStatus, b.getFeatureFlagInfo())
+		})
+	}
+}
+
 func setupSuccessfulMockExecutor(
 	t *testing.T,
 	prepareFn func(options ExecutorPrepareOptions) error,
@@ -1535,10 +1788,13 @@ func setupSuccessfulMockExecutor(
 	executor.On("Run", matchBuildStage(BuildStageDownloadArtifacts)).Return(nil).Once()
 	executor.On("Run", matchBuildStage("step_script")).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageAfterScript)).Return(nil).Once()
-	executor.On("Run", matchBuildStage(BuildStageArchiveCache)).Return(nil).Once()
+	executor.On("Run", matchBuildStage(BuildStageArchiveOnSuccessCache)).Return(nil).Once()
 	executor.On("Run", matchBuildStage(BuildStageUploadOnSuccessArtifacts)).
 		Return(nil).
 		Once()
+	executor.On("Run", matchBuildStage(BuildStageCleanup)).
+		Return(nil).
+		Once()
 
 	return provider, assertFn
 }
@@ -1581,3 +1837,230 @@ func runSuccessfulMockBuild(t *testing.T, prepareFn func(options ExecutorPrepare
 
 	return build
 }
+
+func TestSecretsResolving(t *testing.T) {
+	exampleVariables := JobVariables{
+		{Key: "key", Value: "value"},
+	}
+
+	setupFailureExecutorMocks := func(t *testing.T) (*MockExecutorProvider, func()) {
+		e := new(MockExecutor)
+		p := new(MockExecutorProvider)
+
+		p.On("CanCreate").Return(true).Once()
+		p.On("GetDefaultShell").Return("bash").Once()
+		p.On("GetFeatures", mock.Anything).Return(nil).Once()
+
+		assertFn := func() {
+			e.AssertExpectations(t)
+			p.AssertExpectations(t)
+		}
+
+		return p, assertFn
+	}
+
+	secrets := Secrets{
+		"TEST_SECRET": Secret{
+			Vault: &VaultSecret{},
+		},
+	}
+
+	tests := map[string]struct {
+		secrets                 Secrets
+		resolverCreationError   error
+		prepareExecutorProvider func(t *testing.T) (*MockExecutorProvider, func())
+		returnVariables         JobVariables
+		resolvingError          error
+		expectedVariables       JobVariables
+		expectedError           error
+	}{
+		"secrets not present": {
+			prepareExecutorProvider: func(t *testing.T) (*MockExecutorProvider, func()) {
+				return setupSuccessfulMockExecutor(t, func(options ExecutorPrepareOptions) error { return nil })
+			},
+			expectedError: nil,
+		},
+		"error on creating resolver": {
+			secrets:                 secrets,
+			resolverCreationError:   assert.AnError,
+			prepareExecutorProvider: setupFailureExecutorMocks,
+			expectedError:           assert.AnError,
+		},
+		"error on secrets resolving": {
+			secrets:                 secrets,
+			prepareExecutorProvider: setupFailureExecutorMocks,
+			returnVariables:         exampleVariables,
+			resolvingError:          assert.AnError,
+			expectedVariables:       nil,
+			expectedError:           assert.AnError,
+		},
+		"secrets resolved": {
+			secrets: secrets,
+			prepareExecutorProvider: func(t *testing.T) (*MockExecutorProvider, func()) {
+				return setupSuccessfulMockExecutor(t, func(options ExecutorPrepareOptions) error { return nil })
+			},
+			returnVariables:   exampleVariables,
+			resolvingError:    nil,
+			expectedVariables: exampleVariables,
+			expectedError:     nil,
+		},
+	}
+
+	for tn, tt := range tests {
+		t.Run(tn, func(t *testing.T) {
+			secretsResolverMock := new(MockSecretsResolver)
+			defer secretsResolverMock.AssertExpectations(t)
+
+			p, assertFn := tt.prepareExecutorProvider(t)
+			defer assertFn()
+
+			RegisterExecutorProvider(t.Name(), p)
+
+			successfulBuild, err := GetSuccessfulBuild()
+			require.NoError(t, err)
+
+			successfulBuild.Secrets = tt.secrets
+
+			if tt.resolverCreationError == nil && tt.secrets != nil {
+				secretsResolverMock.On("Resolve", tt.secrets).
+					Return(tt.returnVariables, tt.resolvingError).
+					Once()
+			}
+
+			rc := new(RunnerConfig)
+			rc.RunnerSettings.Executor = t.Name()
+
+			build, err := NewBuild(successfulBuild, rc, nil, nil)
+			assert.NoError(t, err)
+
+			build.secretsResolver = func(_ logger, _ SecretResolverRegistry) (SecretsResolver, error) {
+				return secretsResolverMock, tt.resolverCreationError
+			}
+
+			err = build.Run(&Config{}, &Trace{Writer: os.Stdout})
+
+			assert.Equal(t, tt.expectedVariables, build.secretsVariables)
+
+			if tt.expectedError != nil {
+				assert.ErrorAs(t, err, &tt.expectedError)
+				return
+			}
+			assert.NoError(t, err)
+		})
+	}
+}
+
+func TestBuildSupportedFailureReasons(t *testing.T) {
+	supportedReason := JobFailureReason("supported")
+	unsupportedReason := JobFailureReason("unsupported")
+
+	tests := map[string]struct {
+		supported      []JobFailureReason
+		reason         JobFailureReason
+		expectedReason JobFailureReason
+	}{
+		"empty list with widely supported reason": {
+			supported:      nil,
+			reason:         ScriptFailure,
+			expectedReason: ScriptFailure,
+		},
+		"empty list with unsupported reason": {
+			supported:      nil,
+			reason:         unsupportedReason,
+			expectedReason: UnknownFailure,
+		},
+		"populated list with widely supported reason": {
+			supported:      []JobFailureReason{supportedReason},
+			reason:         ScriptFailure,
+			expectedReason: ScriptFailure,
+		},
+		"populated list with supported reason": {
+			supported:      []JobFailureReason{supportedReason},
+			reason:         supportedReason,
+			expectedReason: supportedReason,
+		},
+		"populated list with unsupported reason": {
+			supported:      []JobFailureReason{supportedReason},
+			reason:         unsupportedReason,
+			expectedReason: UnknownFailure,
+		},
+	}
+
+	for tn, tc := range tests {
+		t.Run(tn, func(t *testing.T) {
+			b := &Build{
+				Runner: &RunnerConfig{},
+				JobResponse: JobResponse{
+					Features: GitlabFeatures{
+						FailureReasons: tc.supported,
+					},
+				},
+			}
+			b.logger = NewBuildLogger(nil, b.Log())
+
+			err := &BuildError{
+				FailureReason: tc.reason,
+			}
+
+			trace := new(MockJobTrace)
+			defer trace.AssertExpectations(t)
+			trace.On(
+				"Fail",
+				err,
+				JobFailureData{
+					Reason:   tc.expectedReason,
+					ExitCode: 0,
+				},
+			).Once()
+
+			b.setTraceStatus(trace, err)
+		})
+	}
+}
+
+func TestSetTraceStatus(t *testing.T) {
+	tests := map[string]struct {
+		err    error
+		assert func(*testing.T, *MockJobTrace, error)
+	}{
+		"nil error is successful": {
+			err: nil,
+			assert: func(t *testing.T, mt *MockJobTrace, err error) {
+				mt.On("Success").Once()
+			},
+		},
+		"build error, script failure": {
+			err: &BuildError{FailureReason: ScriptFailure},
+			assert: func(t *testing.T, mt *MockJobTrace, err error) {
+				mt.On("Fail", err, JobFailureData{Reason: ScriptFailure}).Once()
+			},
+		},
+		"build error, wrapped script failure": {
+			err: fmt.Errorf("wrapped: %w", &BuildError{FailureReason: ScriptFailure}),
+			assert: func(t *testing.T, mt *MockJobTrace, err error) {
+				mt.On("Fail", err, JobFailureData{Reason: ScriptFailure}).Once()
+			},
+		},
+		"non-build error": {
+			err: fmt.Errorf("some error"),
+			assert: func(t *testing.T, mt *MockJobTrace, err error) {
+				mt.On("Fail", err, JobFailureData{Reason: RunnerSystemFailure}).Once()
+			},
+		},
+	}
+
+	for tn, tc := range tests {
+		t.Run(tn, func(t *testing.T) {
+			b := &Build{
+				Runner: &RunnerConfig{},
+			}
+			b.logger = NewBuildLogger(nil, b.Log())
+
+			trace := new(MockJobTrace)
+			defer trace.AssertExpectations(t)
+
+			tc.assert(t, trace, tc.err)
+			b.setTraceStatus(trace, tc.err)
+		})
+	}
+}
