@@ -1,13 +1,14 @@
+//go:build integration
+// +build integration
+
 package custom_test
 
 import (
-	"errors"
 	"fmt"
 	"io/ioutil"
 	"os"
 	"os/exec"
 	"path/filepath"
-	"runtime"
 	"testing"
 	"time"
 
@@ -22,45 +23,34 @@ import (
 	"gitlab.com/gitlab-org/gitlab-runner/shells/shellstest"
 )
 
-var testExecutorFile string
+var (
+	testExecutorFile string
+	testJobInfo      = common.JobInfo{
+		Name:        "test job",
+		Stage:       "test",
+		ProjectID:   0,
+		ProjectName: "test project",
+	}
+)
 
 func TestMain(m *testing.M) {
+	code := 1
+	defer func() {
+		os.Exit(code)
+	}()
+
 	fmt.Println("Compiling test executor")
 
-	curDir, err := os.Getwd()
-	if err != nil {
-		panic("Error on getting the working directory")
-	}
-
-	sourcesDir := filepath.Join(curDir, "testdata", "test_executor")
-	sourcesFile := filepath.Join(sourcesDir, "main.go")
-
 	targetDir, err := ioutil.TempDir("", "test_executor")
 	if err != nil {
 		panic("Error on preparing tmp directory for test executor binary")
 	}
+
 	testExecutorFile = filepath.Join(targetDir, "main")
+	testExecutorFile = buildtest.MustBuildBinary("testdata/test_executor/main.go", testExecutorFile)
+	defer os.RemoveAll(targetDir)
 
-	if runtime.GOOS == "windows" {
-		// Adding it here, explicitly, in if, to show that this OS
-		// requires a special treatment...
-		testExecutorFile += ".exe"
-	}
-
-	cmd := exec.Command("go", "build", "-o", testExecutorFile, sourcesFile)
-	cmd.Stdout = os.Stdout
-	cmd.Stderr = os.Stderr
-
-	fmt.Printf("Executing: %v", cmd)
-	fmt.Println()
-
-	err = cmd.Run()
-	if err != nil {
-		panic("Error on executing go build to prepare test custom executor")
-	}
-
-	code := m.Run()
-	os.Exit(code)
+	code = m.Run()
 }
 
 func newBuild(t *testing.T, jobResponse common.JobResponse, shell string) (*common.Build, func()) {
@@ -69,6 +59,8 @@ func newBuild(t *testing.T, jobResponse common.JobResponse, shell string) (*comm
 
 	t.Log("Build directory:", dir)
 
+	jobResponse.JobInfo = testJobInfo
+
 	build := &common.Build{
 		JobResponse: jobResponse,
 		Runner: &common.RunnerConfig{
@@ -125,6 +117,23 @@ func TestBuildSuccess(t *testing.T) {
 	})
 }
 
+func TestBuildScriptSections(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		if shell == "cmd" || shell == "pwsh" || shell == "powershell" {
+			// support for pwsh and powershell tracked in https://gitlab.com/gitlab-org/gitlab-runner/-/issues/28119
+			t.Skip("CMD, pwsh, powershell not supported")
+		}
+		successfulBuild, err := common.GetSuccessfulBuild()
+		require.NoError(t, err)
+
+		build, cleanup := newBuild(t, successfulBuild, shell)
+		defer cleanup()
+
+		require.NoError(t, err)
+		buildtest.RunBuildWithSections(t, build)
+	})
+}
+
 func TestBuildSuccessRawVariable(t *testing.T) {
 	tests := map[string]struct {
 		command string
@@ -182,7 +191,8 @@ func TestBuildBuildFailure(t *testing.T) {
 		err = buildtest.RunBuild(t, build)
 		assert.Error(t, err)
 		var buildErr *common.BuildError
-		assert.True(t, errors.As(err, &buildErr), "expected %T, got %T", buildErr, err)
+		assert.ErrorAs(t, err, &buildErr)
+		assert.Equal(t, command.BuildFailureExitCode, buildErr.ExitCode)
 	})
 }
 
@@ -203,7 +213,7 @@ func TestBuildSystemFailure(t *testing.T) {
 		err = buildtest.RunBuild(t, build)
 		assert.Error(t, err)
 		var exitError *exec.ExitError
-		assert.True(t, errors.As(err, &exitError), "expected %T, got %T", exitError, err)
+		assert.ErrorAs(t, err, &exitError)
 		t.Log(err)
 	})
 }
@@ -225,48 +235,37 @@ func TestBuildUnknownFailure(t *testing.T) {
 		err = buildtest.RunBuild(t, build)
 		assert.Error(t, err)
 		var errUnknownFailure *command.ErrUnknownFailure
-		assert.True(t, errors.As(err, &errUnknownFailure), "expected %T, got %T", errUnknownFailure, err)
-	})
-}
-
-func TestBuildAbort(t *testing.T) {
-	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
-		longRunningBuild, err := common.GetLongRunningBuild()
-		require.NoError(t, err)
-
-		build, cleanup := newBuild(t, longRunningBuild, shell)
-		defer cleanup()
-
-		abortTimer := time.AfterFunc(time.Second, func() {
-			t.Log("Interrupt")
-			build.SystemInterrupt <- os.Interrupt
-		})
-		defer abortTimer.Stop()
-
-		err = buildtest.RunBuild(t, build)
-		assert.EqualError(t, err, "aborted: interrupt")
+		assert.ErrorAs(t, err, &errUnknownFailure)
+		assert.Equal(t, 255, errUnknownFailure.ExitCode)
 	})
 }
 
 func TestBuildCancel(t *testing.T) {
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
-		longRunningBuild, err := common.GetLongRunningBuild()
-		require.NoError(t, err)
-
-		build, cleanup := newBuild(t, longRunningBuild, shell)
+		build, cleanup := newBuild(t, common.JobResponse{}, shell)
 		defer cleanup()
 
-		trace := &common.Trace{Writer: os.Stdout}
+		updateSleepForCMD := func(build *common.Build) {
+			if shell != "cmd" {
+				return
+			}
 
-		cancelTimer := time.AfterFunc(2*time.Second, func() {
-			t.Log("Cancel")
-			trace.Cancel()
-		})
-		defer cancelTimer.Stop()
+			resp, err := common.GetRemoteLongRunningBuildWithAfterScriptCMD()
+			require.NoError(t, err)
 
-		err = buildtest.RunBuildWithTrace(t, build, trace)
-		assert.EqualError(t, err, "canceled")
-		assert.IsType(t, &common.BuildError{}, err)
+			build.JobResponse = resp
+		}
+
+		buildtest.RunBuildWithCancel(t, build.Runner, updateSleepForCMD)
+	})
+}
+
+func TestBuildMasking(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		build, cleanup := newBuild(t, common.JobResponse{}, shell)
+		defer cleanup()
+
+		buildtest.RunBuildWithMasking(t, build.Runner, nil)
 	})
 }
 
@@ -279,6 +278,7 @@ func TestBuildWithGitStrategyCloneWithoutLFS(t *testing.T) {
 		defer cleanup()
 
 		build.Runner.PreCloneScript = "echo pre-clone-script"
+		build.Runner.PostCloneScript = "echo post-clone-script"
 		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "clone"})
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
@@ -290,6 +290,7 @@ func TestBuildWithGitStrategyCloneWithoutLFS(t *testing.T) {
 		assert.Contains(t, out, "Created fresh repository")
 		assert.Regexp(t, "Checking out [a-f0-9]+ as", out)
 		assert.Contains(t, out, "pre-clone-script")
+		assert.Contains(t, out, "post-clone-script")
 	})
 }
 
@@ -302,6 +303,7 @@ func TestBuildWithGitStrategyCloneNoCheckoutWithoutLFS(t *testing.T) {
 		defer cleanup()
 
 		build.Runner.PreCloneScript = "echo pre-clone-script"
+		build.Runner.PostCloneScript = "echo post-clone-script"
 		build.Variables = append(
 			build.Variables,
 			common.JobVariable{Key: "GIT_STRATEGY", Value: "clone"},
@@ -317,6 +319,7 @@ func TestBuildWithGitStrategyCloneNoCheckoutWithoutLFS(t *testing.T) {
 		assert.Contains(t, out, "Created fresh repository")
 		assert.Contains(t, out, "Skipping Git checkout")
 		assert.Contains(t, out, "pre-clone-script")
+		assert.Contains(t, out, "post-clone-script")
 	})
 }
 
@@ -434,6 +437,7 @@ func TestBuildWithGitSSLAndStrategyFetch(t *testing.T) {
 		defer cleanup()
 
 		build.Runner.PreCloneScript = "echo pre-clone-script"
+		build.Runner.PostCloneScript = "echo post-clone-script"
 		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"})
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
@@ -446,6 +450,7 @@ func TestBuildWithGitSSLAndStrategyFetch(t *testing.T) {
 		assert.Contains(t, out, "Fetching changes")
 		assert.Regexp(t, "Checking out [a-f0-9]+ as", out)
 		assert.Contains(t, out, "pre-clone-script")
+		assert.Contains(t, out, "post-clone-script")
 	})
 }
 
@@ -466,7 +471,7 @@ func TestBuildChangesBranchesWhenFetchingRepo(t *testing.T) {
 		build.GitInfo = common.GetLFSGitInfo(build.GitInfo.RepoURL)
 		out, err = buildtest.RunBuildReturningOutput(t, build)
 		assert.NoError(t, err)
-		assert.Contains(t, out, "Checking out 2371dd05 as add-lfs-object...")
+		assert.Contains(t, out, "Checking out c8f2a61d as add-lfs-object...")
 	})
 }
 
@@ -569,3 +574,97 @@ func TestBuildOnCustomDirectory(t *testing.T) {
 		}
 	})
 }
+
+func TestBuildLogLimitExceeded(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		build, cleanup := newBuild(t, common.JobResponse{}, shell)
+		defer cleanup()
+
+		buildtest.RunBuildWithJobOutputLimitExceeded(t, build.Runner, nil)
+	})
+}
+
+func TestBuildWithAccessToJobResponseFile(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		successfulBuild, err := common.GetSuccessfulBuild()
+		require.NoError(t, err)
+
+		build, cleanup := newBuild(t, successfulBuild, shell)
+		defer cleanup()
+
+		output, err := buildtest.RunBuildReturningOutput(t, build)
+		require.NoError(t, err)
+
+		assert.Contains(t, output, "job ID           => 0")
+		assert.Contains(t, output, fmt.Sprintf("job name         => %s", testJobInfo.Name))
+		assert.Contains(t, output, fmt.Sprintf("job stage        => %s", testJobInfo.Stage))
+		assert.Contains(t, output, fmt.Sprintf("job project ID   => %d", testJobInfo.ProjectID))
+		assert.Contains(t, output, fmt.Sprintf("job project name => %s", testJobInfo.ProjectName))
+	})
+}
+
+func TestCleanupProjectGitClone(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		successfulBuild, err := common.GetSuccessfulBuild()
+		require.NoError(t, err)
+
+		build, cleanup := newBuild(t, successfulBuild, shell)
+		defer cleanup()
+
+		buildtest.RunBuildWithCleanupGitClone(t, build)
+	})
+}
+
+func TestCleanupProjectGitFetch(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		untrackedFilename := "untracked"
+
+		successfulBuild, err := common.GetRemoteBuildResponse(
+			buildtest.GetNewUntrackedFileIntoSubmodulesCommands(untrackedFilename, "", "")...,
+		)
+		require.NoError(t, err)
+		build, cleanup := newBuild(t, successfulBuild, shell)
+		defer cleanup()
+
+		buildtest.RunBuildWithCleanupGitFetch(t, build, untrackedFilename)
+	})
+}
+
+func TestCleanupProjectGitSubmoduleNormal(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		untrackedFile := "untracked"
+		untrackedSubmoduleFile := "untracked_submodule"
+
+		successfulBuild, err := common.GetRemoteBuildResponse(
+			buildtest.GetNewUntrackedFileIntoSubmodulesCommands(untrackedFile, untrackedSubmoduleFile, "")...,
+		)
+		require.NoError(t, err)
+
+		build, cleanup := newBuild(t, successfulBuild, shell)
+		defer cleanup()
+
+		buildtest.RunBuildWithCleanupNormalSubmoduleStrategy(t, build, untrackedFile, untrackedSubmoduleFile)
+	})
+}
+
+func TestCleanupProjectGitSubmoduleRecursive(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		untrackedFile := "untracked"
+		untrackedSubmoduleFile := "untracked_submodule"
+		untrackedSubSubmoduleFile := "untracked_submodule_submodule"
+
+		successfulBuild, err := common.GetRemoteBuildResponse(
+			buildtest.GetNewUntrackedFileIntoSubmodulesCommands(
+				untrackedFile,
+				untrackedSubmoduleFile,
+				untrackedSubSubmoduleFile,
+			)...,
+		)
+
+		require.NoError(t, err)
+		build, cleanup := newBuild(t, successfulBuild, shell)
+		defer cleanup()
+
+		buildtest.RunBuildWithCleanupNormalSubmoduleStrategy(t, build, untrackedFile, untrackedSubmoduleFile)
+	})
+}
