@@ -1,3 +1,4 @@
+//go:build windows
 // +build windows
 
 package permission
@@ -16,6 +17,6 @@ func NewDockerWindowsSetter() Setter {
 }
 
 // Set noop
-func (d dockerWindowsSetter) Set(_ context.Context, _ string) error {
+func (d dockerWindowsSetter) Set(_ context.Context, _ string, _ map[string]string) error {
 	return nil
 }
