@@ -1,3 +1,6 @@
+//go:build !integration
+// +build !integration
+
 package machine
 
 import (
@@ -11,8 +14,7 @@ import (
 )
 
 func TestIfMachineProviderExposesCollectInterface(t *testing.T) {
-	var provider common.ExecutorProvider
-	provider = &machineProvider{}
+	var provider common.ExecutorProvider = &machineProvider{}
 	collector, ok := provider.(prometheus.Collector)
 	assert.True(t, ok)
 	assert.NotNil(t, collector)
