@@ -17,6 +17,7 @@ type machineProvider struct {
 	name        string
 	machine     docker.Machine
 	details     machinesDetails
+	runners     runnersDetails
 	lock        sync.RWMutex
 	acquireLock sync.Mutex
 	// provider stores a real executor that is used to start run the builds
@@ -28,9 +29,20 @@ type machineProvider struct {
 	totalActions      *prometheus.CounterVec
 	currentStatesDesc *prometheus.Desc
 	creationHistogram prometheus.Histogram
+	stoppingHistogram prometheus.Histogram
+	removalHistogram  prometheus.Histogram
 }
 
 func (m *machineProvider) machineDetails(name string, acquire bool) *machineDetails {
+	details := m.ensureDetails(name)
+	if acquire {
+		details = m.tryAcquireMachineDetails(details)
+	}
+
+	return details
+}
+
+func (m *machineProvider) ensureDetails(name string) *machineDetails {
 	m.lock.Lock()
 	defer m.lock.Unlock()
 
@@ -48,62 +60,91 @@ func (m *machineProvider) machineDetails(name string, acquire bool) *machineDeta
 		m.details[name] = details
 	}
 
-	if acquire {
-		if details.isUsed() {
-			return nil
-		}
-		details.State = machineStateAcquired
+	return details
+}
+
+var errNoConfig = errors.New("no runner config specified")
+
+func (m *machineProvider) runnerMachinesCoordinator(config *common.RunnerConfig) (*runnerMachinesCoordinator, error) {
+	if config == nil {
+		return nil, errNoConfig
 	}
 
-	return details
+	m.lock.Lock()
+	defer m.lock.Unlock()
+
+	details, ok := m.runners[config.GetToken()]
+	if !ok {
+		details = newRunnerMachinesCoordinator()
+		m.runners[config.GetToken()] = details
+	}
+
+	return details, nil
 }
 
 func (m *machineProvider) create(config *common.RunnerConfig, state machineState) (*machineDetails, chan error) {
 	name := newMachineName(config)
 	details := m.machineDetails(name, true)
+	m.lock.Lock()
 	details.State = machineStateCreating
 	details.UsedCount = 0
 	details.RetryCount = 0
 	details.LastSeen = time.Now()
+	m.lock.Unlock()
 	errCh := make(chan error, 1)
 
-	// Create machine asynchronously
-	go func() {
-		started := time.Now()
-		err := m.machine.Create(config.Machine.MachineDriver, details.Name, config.Machine.MachineOptions...)
-		for i := 0; i < 3 && err != nil; i++ {
-			details.RetryCount++
-			logrus.WithField("name", details.Name).
-				WithError(err).
-				Warningln("Machine creation failed, trying to provision")
-			time.Sleep(provisionRetryInterval)
-			err = m.machine.Provision(details.Name)
-		}
-
-		if err != nil {
-			logrus.WithField("name", details.Name).
-				WithField("time", time.Since(started)).
-				WithError(err).
-				Errorln("Machine creation failed")
-			_ = m.remove(details.Name, "Failed to create")
-		} else {
-			details.State = state
-			details.Used = time.Now()
-			creationTime := time.Since(started)
-			logrus.WithField("duration", creationTime).
-				WithField("name", details.Name).
-				WithField("now", time.Now()).
-				WithField("retries", details.RetryCount).
-				Infoln("Machine created")
-			m.totalActions.WithLabelValues("created").Inc()
-			m.creationHistogram.Observe(creationTime.Seconds())
-		}
+	// Create machine with the required configuration asynchronously
+	coordinator, err := m.runnerMachinesCoordinator(config)
+	if err != nil {
 		errCh <- err
-	}()
+		return nil, errCh
+	}
+
+	go coordinator.waitForGrowthCapacity(config.Machine.MaxGrowthRate, func() {
+		m.createWithGrowthCapacity(coordinator, config, details, state, errCh)
+	})
 
 	return details, errCh
 }
 
+func (m *machineProvider) createWithGrowthCapacity(
+	coordinator *runnerMachinesCoordinator,
+	config *common.RunnerConfig,
+	details *machineDetails,
+	state machineState,
+	errCh chan error,
+) {
+	logger := logrus.WithField("name", details.Name)
+	started := time.Now()
+
+	err := m.machine.Create(config.Machine.MachineDriver, details.Name, config.Machine.MachineOptions...)
+	if err != nil {
+		logger.WithField("time", time.Since(started)).
+			WithError(err).
+			Errorln("Machine creation failed")
+		_ = m.remove(details.Name, "Failed to create")
+	} else {
+		m.lock.Lock()
+		details.State = state
+		details.Used = time.Now()
+		m.lock.Unlock()
+		creationTime := time.Since(started)
+		m.lock.RLock()
+		logger.WithField("duration", creationTime).
+			WithField("now", time.Now()).
+			WithField("retries", details.RetryCount).
+			Infoln("Machine created")
+		m.lock.RUnlock()
+		m.totalActions.WithLabelValues("created").Inc()
+		m.creationHistogram.Observe(creationTime.Seconds())
+
+		// Signal that a new machine is available. When there's contention, there's no guarantee between the
+		// ordering of reading from errCh and the availability check.
+		coordinator.addAvailableMachine()
+	}
+	errCh <- err
+}
+
 func (m *machineProvider) findFreeMachine(skipCache bool, machines ...string) (details *machineDetails) {
 	// Enumerate all machines in reverse order, to always take the newest machines first
 	for idx := range machines {
@@ -125,18 +166,80 @@ func (m *machineProvider) findFreeMachine(skipCache bool, machines ...string) (d
 	return nil
 }
 
-func (m *machineProvider) useMachine(config *common.RunnerConfig) (details *machineDetails, err error) {
+func (m *machineProvider) findFreeExistingMachine(config *common.RunnerConfig) (*machineDetails, error) {
 	machines, err := m.loadMachines(config)
 	if err != nil {
-		return
+		return nil, err
 	}
-	details = m.findFreeMachine(true, machines...)
-	if details == nil {
-		var errCh chan error
-		details, errCh = m.create(config, machineStateAcquired)
-		err = <-errCh
+
+	return m.findFreeMachine(true, machines...), nil
+}
+
+func (m *machineProvider) useMachine(config *common.RunnerConfig) (*machineDetails, error) {
+	details, err := m.findFreeExistingMachine(config)
+	if err != nil || details != nil {
+		return details, err
 	}
-	return
+
+	return m.createAndAcquireMachine(config)
+}
+
+func (m *machineProvider) createAndAcquireMachine(config *common.RunnerConfig) (*machineDetails, error) {
+	coordinator, err := m.runnerMachinesCoordinator(config)
+	if err != nil {
+		return nil, err
+	}
+
+	newDetails, errCh := m.create(config, machineStateIdle)
+	// Use either a free machine, or the created machine; whichever comes first. There's no guarantee that the created
+	// machine can be used by us because between the time the machine is created, and the acquisition of the machine,
+	// another goroutine may have found it via findFreeMachine and acquired it.
+	var details *machineDetails
+	for details == nil && err == nil {
+		select {
+		case err = <-errCh:
+			if err != nil {
+				return nil, err
+			}
+
+			details = m.tryAcquireMachineDetails(newDetails)
+		case <-coordinator.availableMachineSignal():
+			// Even though the signal is fired and we are *almost* sure that
+			// there's a machine available, let's use the getAvailableMachine
+			// method so that the internal counter is synchonized with what
+			// we are actually doing and so that we can be sure that no other
+			// goroutine that didn't accept the signal and instead used the ticker
+			// hasn't already snatched a machine
+			details, err = m.tryGetFreeExistingMachineFromCoordinator(config, coordinator)
+		case <-time.After(time.Second):
+			details, err = m.tryGetFreeExistingMachineFromCoordinator(config, coordinator)
+		}
+	}
+
+	return details, err
+}
+
+func (m *machineProvider) tryGetFreeExistingMachineFromCoordinator(
+	config *common.RunnerConfig,
+	coordinator *runnerMachinesCoordinator,
+) (*machineDetails, error) {
+	if coordinator.getAvailableMachine() {
+		return m.findFreeExistingMachine(config)
+	}
+
+	return nil, nil
+}
+
+func (m *machineProvider) tryAcquireMachineDetails(details *machineDetails) *machineDetails {
+	m.lock.Lock()
+	defer m.lock.Unlock()
+
+	if details.isUsed() {
+		return nil
+	}
+
+	details.State = machineStateAcquired
+	return details
 }
 
 func (m *machineProvider) retryUseMachine(config *common.RunnerConfig) (details *machineDetails, err error) {
@@ -164,18 +267,20 @@ func (m *machineProvider) removeMachine(details *machineDetails) (err error) {
 		defer m.stuckRemoveLock.Unlock()
 	}
 
-	details.logger().
-		Warningln("Stopping machine")
-	err = m.machine.Stop(details.Name, machineStopCommandTimeout)
+	details.logger().Warningln("Stopping machine")
+	err = runHistogramCountedOperation(m.stoppingHistogram, func() error {
+		return m.machine.Stop(details.Name, machineStopCommandTimeout)
+	})
 	if err != nil {
 		details.logger().
 			WithError(err).
 			Warningln("Error while stopping machine")
 	}
 
-	details.logger().
-		Warningln("Removing machine")
-	err = m.machine.Remove(details.Name)
+	details.logger().Warningln("Removing machine")
+	err = runHistogramCountedOperation(m.removalHistogram, func() error {
+		return m.machine.Remove(details.Name)
+	})
 	if err != nil {
 		details.RetryCount++
 		time.Sleep(removeRetryInterval)
@@ -185,6 +290,14 @@ func (m *machineProvider) removeMachine(details *machineDetails) (err error) {
 	return nil
 }
 
+func runHistogramCountedOperation(histogram prometheus.Histogram, operation func() error) error {
+	startedAt := time.Now()
+	err := operation()
+	histogram.Observe(time.Since(startedAt).Seconds())
+
+	return err
+}
+
 func (m *machineProvider) finalizeRemoval(details *machineDetails) {
 	for {
 		err := m.removeMachine(details)
@@ -229,34 +342,6 @@ func (m *machineProvider) remove(machineName string, reason ...interface{}) erro
 	return nil
 }
 
-func (m *machineProvider) updateMachine(
-	config *common.RunnerConfig,
-	data *machinesData,
-	details *machineDetails,
-) error {
-	if details.State != machineStateIdle {
-		return nil
-	}
-
-	if config.Machine.MaxBuilds > 0 && details.UsedCount >= config.Machine.MaxBuilds {
-		// Limit number of builds
-		return errors.New("too many builds")
-	}
-
-	if data.Total() >= config.Limit && config.Limit > 0 {
-		// Limit maximum number of machines
-		return errors.New("too many machines")
-	}
-
-	if time.Since(details.Used) > time.Second*time.Duration(config.Machine.GetIdleTime()) {
-		if data.Idle >= config.Machine.GetIdleCount() {
-			// Remove machine that are way over the idle time
-			return errors.New("too many idle machines")
-		}
-	}
-	return nil
-}
-
 func (m *machineProvider) updateMachines(
 	machines []string,
 	config *common.RunnerConfig,
@@ -268,11 +353,11 @@ func (m *machineProvider) updateMachines(
 		details := m.machineDetails(name, false)
 		details.LastSeen = time.Now()
 
-		err := m.updateMachine(config, &data, details)
-		if err == nil {
+		reason := shouldRemoveIdle(config, &data, details)
+		if reason == dontRemoveIdleMachine {
 			validMachines = append(validMachines, name)
 		} else {
-			_ = m.remove(details.Name, err)
+			_ = m.remove(details.Name, reason)
 		}
 
 		data.Add(details)
@@ -280,17 +365,15 @@ func (m *machineProvider) updateMachines(
 	return
 }
 
+// createMachines starts goroutines that are creating the new machines.
+// Limiting strategy is used to ensure the autoscaling parameters are respected.
 func (m *machineProvider) createMachines(config *common.RunnerConfig, data *machinesData) {
-	// Create a new machines and mark them as Idle
 	for {
-		if data.Available() >= config.Machine.GetIdleCount() {
-			// Limit maximum number of idle machines
-			break
-		}
-		if data.Total() >= config.Limit && config.Limit > 0 {
-			// Limit maximum number of machines
-			break
+		if !canCreateIdle(config, data) {
+			return
 		}
+
+		// Create a new machine and mark it as Idle
 		m.create(config, machineStateIdle)
 		data.Creating++
 	}
@@ -362,12 +445,15 @@ func (m *machineProvider) Acquire(config *common.RunnerConfig) (common.ExecutorD
 	// Pre-create machines
 	m.createMachines(config, &machinesData)
 
-	logrus.WithFields(machinesData.Fields()).
+	logger := logrus.WithFields(machinesData.Fields()).
 		WithField("runner", config.ShortDescription()).
-		WithField("minIdleCount", config.Machine.GetIdleCount()).
+		WithField("idleCountMin", config.Machine.GetIdleCountMin()).
+		WithField("idleCount", config.Machine.GetIdleCount()).
+		WithField("idleScaleFactor", config.Machine.GetIdleScaleFactor()).
 		WithField("maxMachines", config.Limit).
-		WithField("time", time.Now()).
-		Debugln("Docker Machine Details")
+		WithField("maxMachineCreate", config.Machine.MaxGrowthRate)
+
+	logger.WithField("time", time.Now()).Debugln("Docker Machine Details")
 	machinesData.writeDebugInformation()
 
 	// Try to find a free machine
@@ -376,11 +462,13 @@ func (m *machineProvider) Acquire(config *common.RunnerConfig) (common.ExecutorD
 		return details, nil
 	}
 
-	// If we have a free machines we can process a build
-	if config.Machine.GetIdleCount() != 0 && machinesData.Idle == 0 {
-		err = errors.New("no free machines that can process builds")
+	if config.Machine.GetIdleCount() == 0 {
+		logger.Info("IdleCount is set to 0 so the machine will be created on demand in job context")
+	} else if machinesData.Idle == 0 {
+		return nil, &common.NoFreeExecutorError{Message: "no free machines that can process builds"}
 	}
-	return nil, err
+
+	return nil, nil
 }
 
 //nolint:nakedret
@@ -429,22 +517,46 @@ func (m *machineProvider) Use(
 func (m *machineProvider) Release(config *common.RunnerConfig, data common.ExecutorData) {
 	// Release machine
 	details, ok := data.(*machineDetails)
-	if ok {
-		// Mark last used time when is Used
-		if details.State == machineStateUsed {
-			details.Used = time.Now()
-		}
-
-		// Remove machine if we already used it
-		if config != nil && config.Machine != nil &&
-			config.Machine.MaxBuilds > 0 && details.UsedCount >= config.Machine.MaxBuilds {
-			err := m.remove(details.Name, "Too many builds")
-			if err == nil {
-				return
-			}
-		}
-		details.State = machineStateIdle
+	if !ok {
+		return
 	}
+
+	m.lock.Lock()
+	// Mark last used time when is Used
+	if details.State == machineStateUsed {
+		details.Used = time.Now()
+	}
+	m.lock.Unlock()
+
+	// Remove machine if we already used it
+	if config != nil && config.Machine != nil &&
+		config.Machine.MaxBuilds > 0 && details.UsedCount >= config.Machine.MaxBuilds {
+		err := m.remove(details.Name, "Too many builds")
+		if err == nil {
+			return
+		}
+	}
+	m.lock.Lock()
+	details.State = machineStateIdle
+	m.lock.Unlock()
+
+	// Signal pending builds that a new machine is available.
+	if err := m.signalRelease(config); err != nil {
+		return
+	}
+}
+
+func (m *machineProvider) signalRelease(config *common.RunnerConfig) error {
+	coordinator, err := m.runnerMachinesCoordinator(config)
+	if err != nil && err != errNoConfig {
+		return err
+	}
+
+	if err != errNoConfig && coordinator != nil {
+		coordinator.addAvailableMachine()
+	}
+
+	return nil
 }
 
 func (m *machineProvider) CanCreate() bool {
@@ -455,6 +567,10 @@ func (m *machineProvider) GetFeatures(features *common.FeaturesInfo) error {
 	return m.provider.GetFeatures(features)
 }
 
+func (m *machineProvider) GetConfigInfo(input *common.RunnerConfig, output *common.ConfigInfo) {
+	m.provider.GetConfigInfo(input, output)
+}
+
 func (m *machineProvider) GetDefaultShell() string {
 	return m.provider.GetDefaultShell()
 }
@@ -474,6 +590,7 @@ func newMachineProvider(name, executor string) *machineProvider {
 	return &machineProvider{
 		name:     name,
 		details:  make(machinesDetails),
+		runners:  make(runnersDetails),
 		machine:  docker.NewMachineCommand(),
 		provider: provider,
 		totalActions: prometheus.NewCounterVec(
@@ -504,5 +621,25 @@ func newMachineProvider(name, executor string) *machineProvider {
 				},
 			},
 		),
+		stoppingHistogram: prometheus.NewHistogram(
+			prometheus.HistogramOpts{
+				Name:    "gitlab_runner_autoscaling_machine_stopping_duration_seconds",
+				Help:    "Histogram of machine stopping time.",
+				Buckets: []float64{1, 3, 5, 10, 30, 50, 60, 80, 90, 120},
+				ConstLabels: prometheus.Labels{
+					"executor": name,
+				},
+			},
+		),
+		removalHistogram: prometheus.NewHistogram(
+			prometheus.HistogramOpts{
+				Name:    "gitlab_runner_autoscaling_machine_removal_duration_seconds",
+				Help:    "Histogram of machine removal time.",
+				Buckets: []float64{1, 3, 5, 10, 30, 50, 60, 80, 90, 120},
+				ConstLabels: prometheus.Labels{
+					"executor": name,
+				},
+			},
+		),
 	}
 }
