@@ -1,3 +1,6 @@
+//go:build !integration
+// +build !integration
+
 package machine
 
 import (
@@ -5,10 +8,14 @@ import (
 	"fmt"
 	"strings"
 	"sync"
+	"sync/atomic"
 	"testing"
 	"time"
 
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/mock"
+	"github.com/stretchr/testify/require"
+
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/docker"
 )
@@ -58,8 +65,8 @@ var machineNoConnect = &common.RunnerConfig{
 	},
 }
 
-func createMachineConfig(idleCount int, idleTime int) *common.RunnerConfig {
-	return &common.RunnerConfig{
+func createMachineConfig(t *testing.T, idleCount int, idleTime int) *common.RunnerConfig {
+	conf := &common.RunnerConfig{
 		RunnerSettings: common.RunnerSettings{
 			Machine: &common.DockerMachine{
 				MachineName: "test-machine-%s",
@@ -68,21 +75,9 @@ func createMachineConfig(idleCount int, idleTime int) *common.RunnerConfig {
 			},
 		},
 	}
-}
-
-func createMachineOffPeakIdleConfig(offPeakPeriod string) *common.RunnerConfig {
-	return &common.RunnerConfig{
-		RunnerSettings: common.RunnerSettings{
-			Machine: &common.DockerMachine{
-				MachineName:      "test-machine-%s",
-				IdleCount:        2,
-				IdleTime:         0,
-				OffPeakIdleCount: 0,
-				OffPeakIdleTime:  0,
-				OffPeakPeriods:   []string{offPeakPeriod},
-			},
-		},
-	}
+	err := conf.RunnerSettings.Machine.CompilePeriods()
+	require.NoError(t, err)
+	return conf
 }
 
 type testMachine struct {
@@ -103,10 +98,10 @@ func (m *testMachine) Create(driver, name string, opts ...string) error {
 	if strings.Contains(name, "second-fail") {
 		if !m.second {
 			m.second = true
-			return errors.New("Failed to create")
+			return errors.New("failed to create")
 		}
 	} else if strings.Contains(name, "create-fail") || strings.Contains(name, "provision-fail") {
-		return errors.New("Failed to create")
+		return errors.New("failed to create")
 	}
 	m.machines = append(m.machines, name)
 	m.Created <- true
@@ -119,7 +114,7 @@ func (m *testMachine) Provision(name string) error {
 	defer m.mutex.Unlock()
 
 	if strings.Contains(name, "provision-fail") || strings.Contains(name, "second-fail") {
-		return errors.New("Failed to provision")
+		return errors.New("failed to provision")
 	}
 	m.machines = append(m.machines, name)
 	return nil
@@ -169,16 +164,13 @@ func (m *testMachine) List() (machines []string, err error) {
 	return m.machines, nil
 }
 
-func (m *testMachine) CanConnect(name string) bool {
-	if strings.Contains(name, "no-can-connect") {
-		return false
-	}
-	return true
+func (m *testMachine) CanConnect(name string, skipCache bool) bool {
+	return !strings.Contains(name, "no-can-connect")
 }
 
-func (m *testMachine) Credentials(name string) (dc docker_helpers.DockerCredentials, err error) {
+func (m *testMachine) Credentials(name string) (dc docker.Credentials, err error) {
 	if strings.Contains(name, "no-connect") {
-		err = errors.New("Failed to connect")
+		err = errors.New("failed to connect")
 	}
 	return
 }
@@ -195,13 +187,13 @@ func countIdleMachines(p *machineProvider) (count int) {
 	return
 }
 
-func assertIdleMachines(t *testing.T, p *machineProvider, expected int, msgAndArgs ...interface{}) bool {
+func assertIdleMachines(t *testing.T, p *machineProvider, expected int, msgAndArgs ...interface{}) {
 	var idle int
 	for i := 0; i < 10; i++ {
 		idle = countIdleMachines(p)
 
 		if expected == idle {
-			return true
+			return
 		}
 
 		time.Sleep(50 * time.Microsecond)
@@ -209,7 +201,6 @@ func assertIdleMachines(t *testing.T, p *machineProvider, expected int, msgAndAr
 
 	result := fmt.Sprintf("should have %d idle, but has %d", expected, idle)
 	assert.Fail(t, result, msgAndArgs...)
-	return false
 }
 
 func countTotalMachines(p *machineProvider) (count int) {
@@ -224,13 +215,13 @@ func countTotalMachines(p *machineProvider) (count int) {
 	return
 }
 
-func assertTotalMachines(t *testing.T, p *machineProvider, expected int, msgAndArgs ...interface{}) bool {
+func assertTotalMachines(t *testing.T, p *machineProvider, expected int, msgAndArgs ...interface{}) {
 	var total int
 	for i := 0; i < 10; i++ {
 		total = countTotalMachines(p)
 
 		if expected == total {
-			return true
+			return
 		}
 
 		time.Sleep(50 * time.Microsecond)
@@ -238,7 +229,6 @@ func assertTotalMachines(t *testing.T, p *machineProvider, expected int, msgAndA
 
 	result := fmt.Sprintf("should have %d total, but has %d", expected, total)
 	assert.Fail(t, result, msgAndArgs...)
-	return false
 }
 
 func testMachineProvider(machine ...string) (*machineProvider, *testMachine) {
@@ -276,21 +266,21 @@ func TestMachineDetails(t *testing.T) {
 
 func TestMachineFindFree(t *testing.T) {
 	p, tm := testMachineProvider("no-can-connect")
-	d1 := p.findFreeMachine()
+	d1 := p.findFreeMachine(false)
 	assert.Nil(t, d1, "no machines, return nil")
 
-	d2 := p.findFreeMachine("machine1")
+	d2 := p.findFreeMachine(false, "machine1")
 	assert.NotNil(t, d2, "acquire one machine")
 
-	d3 := p.findFreeMachine("machine1")
+	d3 := p.findFreeMachine(false, "machine1")
 	assert.Nil(t, d3, "fail to acquire that machine")
 
-	d4 := p.findFreeMachine("machine1", "machine2")
+	d4 := p.findFreeMachine(false, "machine1", "machine2")
 	assert.NotNil(t, d4, "acquire a new machine")
 	assert.NotEqual(t, d2, d4, "and it's a different machine")
 
 	assert.Len(t, tm.machines, 1, "has one machine")
-	d5 := p.findFreeMachine("machine1", "no-can-connect")
+	d5 := p.findFreeMachine(false, "machine1", "no-can-connect")
 	assert.Nil(t, d5, "fails to acquire machine to which he can't connect")
 }
 
@@ -305,6 +295,10 @@ func TestMachineCreationAndRemoval(t *testing.T) {
 	assert.Equal(t, 0, d.UsedCount)
 	assert.NotNil(t, p.details[d.Name])
 
+	err := p.remove(d.Name)
+	assert.NoError(t, err)
+	assert.Equal(t, machineStateRemoving, d.State)
+
 	d2, errCh := p.create(machineProvisionFail, machineStateUsed)
 	assert.NotNil(t, d2)
 	assert.Error(t, <-errCh, "Fails, because it fails to provision machine")
@@ -312,11 +306,8 @@ func TestMachineCreationAndRemoval(t *testing.T) {
 
 	d3, errCh := p.create(machineCreateFail, machineStateUsed)
 	assert.NotNil(t, d3)
-	assert.NoError(t, <-errCh)
-	assert.Equal(t, machineStateUsed, d3.State)
-
-	p.remove(d.Name)
-	assert.Equal(t, machineStateRemoving, d.State)
+	assert.Error(t, <-errCh)
+	assert.Equal(t, machineStateRemoving, d3.State)
 }
 
 func TestMachineUse(t *testing.T) {
@@ -340,6 +331,128 @@ func TestMachineUse(t *testing.T) {
 	assert.Error(t, err, "fails to create a new machine")
 }
 
+func TestMachineReuse(t *testing.T) {
+	// Create a machine with an idle state. Then try to create additional ones.
+	// while the creation of the all subsequent ones are blocked use the first one,
+	// making sure that the whole process works
+	machineGrowthConfig := &common.RunnerConfig{
+		RunnerSettings: common.RunnerSettings{
+			Machine: &common.DockerMachine{
+				MachineName:   "growth-temp-%s",
+				MaxGrowthRate: 1,
+				IdleTime:      5,
+			},
+		},
+	}
+
+	p := newMachineProvider("docker+machine", "docker")
+
+	machineMock := &docker.MockMachine{}
+	defer machineMock.AssertExpectations(t)
+	p.machine = machineMock
+
+	var blockCreatingMachineWg sync.WaitGroup
+	blockCreatingMachineWg.Add(1)
+
+	var createdMachineDetails *machineDetails
+
+	machineMock.On("Create", mock.Anything, mock.Anything).
+		Return(nil).
+		Once()
+
+	machineMock.On("Create", mock.Anything, mock.Anything).
+		Run(func(args mock.Arguments) {
+			// Free the previously created machine after the useMachine call has already happened.
+			// The useMachine call tries to create a new machine at first because a free one doesn't exist
+			// however it's blocked by blockCreatingMachineWg, thus it's waiting for us to release a new one.
+			// If useMachine never returns because it can't find a machine then that's a bug.
+			time.AfterFunc(time.Second, func() {
+				p.Release(machineGrowthConfig, createdMachineDetails)
+			})
+
+			blockCreatingMachineWg.Wait()
+		}).
+		Return(nil).
+		Once()
+
+	machineMock.On("CanConnect", mock.Anything, mock.Anything).
+		Return(true).
+		Once()
+
+	createdMachineDetails, errCh := p.create(machineGrowthConfig, machineStateUsed)
+	require.NotNil(t, createdMachineDetails)
+	require.NoError(t, <-errCh)
+
+	machineMock.On("List").Return([]string{createdMachineDetails.Name}, nil)
+
+	usedMachineDetails, err := p.useMachine(machineGrowthConfig)
+	require.NoError(t, err)
+	require.Equal(t, createdMachineDetails, usedMachineDetails)
+}
+
+func TestMachineReuseWithContention(t *testing.T) {
+	// Create machines while trying to reuse them with a contention.
+	// Make sure that there are no deadlocks, data races and that machines
+	// are provided to the caller.
+	machineGrowthConfig := &common.RunnerConfig{
+		RunnerSettings: common.RunnerSettings{
+			Machine: &common.DockerMachine{
+				MachineName:   "growth-temp-%s",
+				MaxGrowthRate: 10,
+				IdleTime:      5,
+			},
+		},
+	}
+
+	p := newMachineProvider("docker+machine", "docker")
+
+	machineMock := &docker.MockMachine{}
+	defer machineMock.AssertExpectations(t)
+	p.machine = machineMock
+
+	var listLock sync.Mutex
+	list := make([]string, 0)
+
+	listCall := machineMock.On("List").Return([]string{}, nil)
+	machineMock.On("Create", mock.Anything, mock.Anything).
+		Run(func(args mock.Arguments) {
+			name := args.String(1)
+
+			listLock.Lock()
+			list = append(list, name)
+			listCopy := make([]string, len(list))
+			copy(listCopy, list)
+			listCall.Return(listCopy, nil).Maybe()
+			listLock.Unlock()
+		}).
+		Return(nil)
+
+	machineMock.On("CanConnect", mock.Anything, mock.Anything).
+		Return(true).
+		Maybe()
+
+	const N = 500
+	var wg sync.WaitGroup
+	wg.Add(N)
+
+	startCh := make(chan struct{})
+	for i := 0; i < N; i++ {
+		go func() {
+			defer wg.Done()
+			<-startCh
+
+			usedMachineDetails, err := p.useMachine(machineGrowthConfig)
+			require.NoError(t, err)
+			require.NotNil(t, usedMachineDetails)
+			p.Release(machineGrowthConfig, usedMachineDetails)
+		}()
+	}
+
+	close(startCh)
+	wg.Wait()
+	assert.NotEmpty(t, list)
+}
+
 func TestMachineTestRetry(t *testing.T) {
 	provisionRetryInterval = 0
 
@@ -353,6 +466,88 @@ func TestMachineTestRetry(t *testing.T) {
 	assert.Equal(t, machineStateAcquired, d1.State)
 }
 
+func TestMachineAcquireGrowthCapacity(t *testing.T) {
+	p, _ := testMachineProvider()
+
+	tests := map[string]struct {
+		maxGrowthCapacity int
+		concurrency       int
+
+		expectedMaxConcurrentCalls int
+	}{
+		"growth capacity 3 concurrency 1": {
+			maxGrowthCapacity: 3,
+			concurrency:       1,
+
+			expectedMaxConcurrentCalls: 1,
+		},
+		"growth capacity 3 concurrency 15": {
+			maxGrowthCapacity: 3,
+			concurrency:       15,
+
+			expectedMaxConcurrentCalls: 3,
+		},
+	}
+
+	for tn, tt := range tests {
+		t.Run(tn, func(t *testing.T) {
+			machineMock := docker.MockMachine{}
+			defer machineMock.AssertExpectations(t)
+			p.machine = &machineMock
+
+			var wg sync.WaitGroup
+			var concurrentCalls, maxConcurrentCalls int32
+			var maxConcurrentCallsLock sync.Mutex
+
+			machineMock.On("Create", mock.Anything, mock.Anything).
+				Run(func(mock.Arguments) {
+					defer atomic.AddInt32(&concurrentCalls, -1)
+					cc := atomic.AddInt32(&concurrentCalls, 1)
+
+					maxConcurrentCallsLock.Lock()
+					if cc > maxConcurrentCalls {
+						maxConcurrentCalls = cc
+					}
+					maxConcurrentCallsLock.Unlock()
+
+					// simulate a network call in order to allow some goroutines to get in
+					// line, otherwise we will never get past 1 concurrent call
+					time.Sleep(300 * time.Millisecond)
+				}).
+				Return(nil)
+
+			signal := make(chan struct{})
+			for i := 0; i < tt.concurrency; i++ {
+				wg.Add(1)
+				go func() {
+					<-signal
+					_, errCh := p.create(&common.RunnerConfig{
+						Limit:             tt.concurrency,
+						RunnerCredentials: common.RunnerCredentials{},
+						RunnerSettings: common.RunnerSettings{
+							Machine: &common.DockerMachine{
+								IdleCount:     tt.concurrency,
+								MachineName:   "test",
+								MaxGrowthRate: tt.maxGrowthCapacity,
+							},
+						},
+					}, machineStateIdle)
+
+					<-errCh
+					wg.Done()
+				}()
+			}
+
+			// wait for all goroutines to fire up and line up for the signal in order to have a fair race
+			// to the Create method
+			close(signal)
+			wg.Wait()
+
+			assert.Equal(t, tt.expectedMaxConcurrentCalls, int(maxConcurrentCalls))
+		})
+	}
+}
+
 func TestMachineAcquireAndRelease(t *testing.T) {
 	p, _ := testMachineProvider("test-machine")
 
@@ -373,7 +568,7 @@ func TestMachineAcquireAndRelease(t *testing.T) {
 func TestMachineOnDemandMode(t *testing.T) {
 	p, _ := testMachineProvider()
 
-	config := createMachineConfig(0, 1)
+	config := createMachineConfig(t, 0, 1)
 	_, err := p.Acquire(config)
 	assert.NoError(t, err)
 }
@@ -381,7 +576,7 @@ func TestMachineOnDemandMode(t *testing.T) {
 func TestMachinePreCreateMode(t *testing.T) {
 	p, m := testMachineProvider()
 
-	config := createMachineConfig(1, 5)
+	config := createMachineConfig(t, 1, 5)
 	d, err := p.Acquire(config)
 	assert.Error(t, err, "it should fail with message that currently there's no free machines")
 	assert.Nil(t, d)
@@ -395,7 +590,7 @@ func TestMachinePreCreateMode(t *testing.T) {
 	p.Release(config, d)
 	assertIdleMachines(t, p, 1, "after releasing it should have one free node")
 
-	config = createMachineConfig(2, 5)
+	config = createMachineConfig(t, 2, 5)
 	d, err = p.Acquire(config)
 	assert.NoError(t, err)
 	p.Release(config, d)
@@ -403,7 +598,7 @@ func TestMachinePreCreateMode(t *testing.T) {
 	<-m.Created
 	assertIdleMachines(t, p, 2, "it should start creating a second machine")
 
-	config = createMachineConfig(1, 0)
+	config = createMachineConfig(t, 1, 0)
 	config.Limit = 1
 	d, err = p.Acquire(config)
 	assert.NoError(t, err)
@@ -425,7 +620,7 @@ func TestMachinePreCreateMode(t *testing.T) {
 func TestMachineLimitMax(t *testing.T) {
 	p, _ := testMachineProvider()
 
-	config := createMachineConfig(10, 5)
+	config := createMachineConfig(t, 10, 5)
 	config.Limit = 5
 
 	d, err := p.Acquire(config)
@@ -435,11 +630,13 @@ func TestMachineLimitMax(t *testing.T) {
 
 	config.Limit = 8
 	d, err = p.Acquire(config)
+	assert.NoError(t, err)
 	p.Release(config, d)
 	assertIdleMachines(t, p, 8, "it should upscale to 8 nodes")
 
 	config.Limit = 2
 	d, err = p.Acquire(config)
+	assert.NoError(t, err)
 	p.Release(config, d)
 	assertIdleMachines(t, p, 2, "it should downscale to 2 nodes")
 }
@@ -448,7 +645,7 @@ func TestMachineMaxBuildsForExistingMachines(t *testing.T) {
 	provisionRetryInterval = 0
 
 	p, _ := testMachineProvider("remove-fail")
-	config := createMachineConfig(1, 5)
+	config := createMachineConfig(t, 1, 5)
 	config.Machine.MaxBuilds = 1
 	d, err := p.Acquire(config)
 	assert.Error(t, err)
@@ -456,7 +653,7 @@ func TestMachineMaxBuildsForExistingMachines(t *testing.T) {
 }
 
 func TestMachineMaxBuilds(t *testing.T) {
-	config := createMachineConfig(1, 5)
+	config := createMachineConfig(t, 1, 5)
 	p, _ := testMachineProvider(newMachineName(config))
 	config.Machine.MaxBuilds = 2 // by default we set it to 1
 	d, err := p.Acquire(config)
@@ -477,7 +674,7 @@ func TestMachineMaxBuilds(t *testing.T) {
 func TestMachineIdleLimits(t *testing.T) {
 	p, _ := testMachineProvider()
 
-	config := createMachineConfig(2, 1)
+	config := createMachineConfig(t, 2, 1)
 	d, errCh := p.create(config, machineStateIdle)
 	assert.NoError(t, <-errCh, "machine creation should not fail")
 
@@ -486,50 +683,18 @@ func TestMachineIdleLimits(t *testing.T) {
 	assert.NoError(t, err)
 	assert.Equal(t, machineStateIdle, d.State, "machine should not be removed, because is still in idle time")
 
-	config = createMachineConfig(2, 0)
+	config = createMachineConfig(t, 2, 0)
 	d3, err := p.Acquire(config)
 	p.Release(config, d3)
 	assert.NoError(t, err)
 	assert.Equal(t, machineStateIdle, d.State, "machine should not be removed, because no more than two idle")
 
-	config = createMachineConfig(0, 0)
+	config = createMachineConfig(t, 0, 0)
 	d4, err := p.Acquire(config)
 	p.Release(config, d4)
 	assert.NoError(t, err)
-	assert.Equal(t, machineStateRemoving, d.State, "machine should not be removed, because no more than two idle")
-	assert.Equal(t, "Too many idle machines", d.Reason)
-}
-
-func TestMachineOffPeakIdleLimits(t *testing.T) {
-	daysOfWeek := map[time.Weekday]string{
-		time.Monday:    "mon",
-		time.Tuesday:   "tue",
-		time.Wednesday: "wed",
-		time.Thursday:  "thu",
-		time.Friday:    "fri",
-		time.Saturday:  "sat",
-		time.Sunday:    "sun",
-	}
-	now := time.Now()
-	offPeakEnabledPeriod := fmt.Sprintf("* * * * * %s *", daysOfWeek[now.Weekday()])
-	offPeakDisabledPeriod := fmt.Sprintf("* * * * * %s *", daysOfWeek[now.Add(time.Hour*48).Weekday()])
-
-	p, _ := testMachineProvider()
-
-	config := createMachineOffPeakIdleConfig(offPeakDisabledPeriod)
-	d, errCh := p.create(config, machineStateIdle)
-	assert.NoError(t, <-errCh, "machine creation should not fail")
-
-	d2, err := p.Acquire(config)
-	assert.NoError(t, err)
-	p.Release(config, d2)
-	assert.Equal(t, machineStateIdle, d.State, "machine should not be removed, because not in OffPeak time mode")
-
-	config = createMachineOffPeakIdleConfig(offPeakEnabledPeriod)
-	d3, err := p.Acquire(config)
-	p.Release(config, d3)
-	assert.NoError(t, err)
-	assert.Equal(t, machineStateRemoving, d.State, "machine should be removed, because in OffPeak time mode")
+	assert.Equal(t, machineStateRemoving, d.State, "machine should be removed, because there are no idle")
+	assert.Equal(t, "too many idle machines", d.Reason)
 }
 
 func TestMachineUseOnDemand(t *testing.T) {
@@ -564,8 +729,7 @@ func TestMachineReleaseIfInvalidDataArePassed(t *testing.T) {
 	assert.NotNil(t, nd)
 	assertTotalMachines(t, p, 1, "it creates one machine")
 
-	err = p.Release(nil, nd)
-	assert.NoError(t, err, "it should not fail")
+	p.Release(nil, nd)
 }
 
 func TestMachineCreationIfFailedToConnect(t *testing.T) {
@@ -575,3 +739,26 @@ func TestMachineCreationIfFailedToConnect(t *testing.T) {
 	assert.Error(t, err, "it create a new machine")
 	assert.Nil(t, nd)
 }
+
+func TestIntermediateMachineList(t *testing.T) {
+	p, _ := testMachineProvider()
+	p.details = machinesDetails{
+		"machine1": &machineDetails{
+			Name:  "machine1",
+			State: machineStateIdle,
+		},
+		"machine2": &machineDetails{
+			Name:  "machine2",
+			State: machineStateCreating,
+		},
+		"machine3": &machineDetails{
+			Name:  "machine3",
+			State: machineStateCreating,
+		},
+	}
+
+	expectedIntermediateMachines := []string{"machine3"}
+
+	intermediateMachine := p.intermediateMachineList([]string{"machine1", "machine2"})
+	assert.Equal(t, expectedIntermediateMachines, intermediateMachine)
+}
