@@ -1,3 +1,6 @@
+//go:build !integration
+// +build !integration
+
 /*
 Copyright 2014 The Kubernetes Authors All rights reserved.
 
@@ -303,7 +306,7 @@ func TestAttachErrorGettingPod(t *testing.T) {
 		Config:        clientConfig,
 	}
 
-	assert.True(t, errors.Is(opts.Run(), err))
+	assert.ErrorIs(t, opts.Run(), err)
 }
 
 func TestAttachPodNotRunning(t *testing.T) {
