@@ -5,6 +5,7 @@ import (
 	"fmt"
 	"io"
 	"net/http"
+	"strings"
 	"time"
 
 	"golang.org/x/net/context"
@@ -16,12 +17,28 @@ import (
 	"k8s.io/client-go/tools/clientcmd"
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
+	"gitlab.com/gitlab-org/gitlab-runner/executors/kubernetes/internal/pull"
 )
 
 type kubeConfigProvider func() (*restclient.Config, error)
 
+type resourceQuantityError struct {
+	resource string
+	value    string
+	inner    error
+}
+
+func (r *resourceQuantityError) Error() string {
+	return fmt.Sprintf("parsing resource %q with value %q: %q", r.resource, r.value, r.inner)
+}
+
+func (r *resourceQuantityError) Is(err error) bool {
+	t, ok := err.(*resourceQuantityError)
+	return ok && r.resource == t.resource && r.value == t.value && r.inner == t.inner
+}
+
 var (
-	// inClusterConfig parses kubernets configuration reading in cluster values
+	// inClusterConfig parses kubernetes configuration reading in cluster values
 	inClusterConfig kubeConfigProvider = restclient.InClusterConfig
 	// defaultKubectlConfig parses kubectl configuration ad loads the default cluster
 	defaultKubectlConfig kubeConfigProvider = loadDefaultKubectlConfig
@@ -126,35 +143,39 @@ type podPhaseResponse struct {
 }
 
 func getPodPhase(c *kubernetes.Clientset, pod *api.Pod, out io.Writer) podPhaseResponse {
-	pod, err := c.CoreV1().Pods(pod.Namespace).Get(pod.Name, metav1.GetOptions{})
+	// TODO: handle the context properly with https://gitlab.com/gitlab-org/gitlab-runner/-/issues/27932
+	pod, err := c.CoreV1().Pods(pod.Namespace).Get(context.TODO(), pod.Name, metav1.GetOptions{})
 	if err != nil {
 		return podPhaseResponse{true, api.PodUnknown, err}
 	}
 
 	ready, err := isRunning(pod)
-
-	if err != nil {
+	if err != nil || ready {
 		return podPhaseResponse{true, pod.Status.Phase, err}
 	}
 
-	if ready {
-		return podPhaseResponse{true, pod.Status.Phase, nil}
-	}
-
 	// check status of containers
-	for _, container := range pod.Status.ContainerStatuses {
+	for _, container := range append(pod.Status.ContainerStatuses, pod.Status.InitContainerStatuses...) {
 		if container.Ready {
 			continue
 		}
-		if container.State.Waiting == nil {
+		waiting := container.State.Waiting
+		if waiting == nil {
 			continue
 		}
 
-		switch container.State.Waiting.Reason {
-		case "ErrImagePull", "ImagePullBackOff", "InvalidImageName":
-			err = fmt.Errorf("image pull failed: %s", container.State.Waiting.Message)
-			err = &common.BuildError{Inner: err}
+		switch waiting.Reason {
+		case "InvalidImageName":
+			err = &common.BuildError{Inner: fmt.Errorf("image pull failed: %s", waiting.Message)}
 			return podPhaseResponse{true, api.PodUnknown, err}
+		case "ErrImagePull", "ImagePullBackOff":
+			msg := fmt.Sprintf("image pull failed: %s", waiting.Message)
+			imagePullErr := &pull.ImagePullError{Message: msg, Image: container.Image}
+			return podPhaseResponse{
+				true,
+				api.PodUnknown,
+				&common.BuildError{Inner: imagePullErr, FailureReason: common.ScriptFailure},
+			}
 		}
 	}
 
@@ -165,6 +186,21 @@ func getPodPhase(c *kubernetes.Clientset, pod *api.Pod, out io.Writer) podPhaseR
 		pod.Name,
 		pod.Status.Phase,
 	)
+
+	for _, condition := range pod.Status.Conditions {
+		// skip conditions with no reason, these are typically expected pod conditions
+		if condition.Reason == "" {
+			continue
+		}
+
+		_, _ = fmt.Fprintf(
+			out,
+			"\t%s: %q\n",
+			condition.Reason,
+			condition.Message,
+		)
+	}
+
 	return podPhaseResponse{false, pod.Status.Phase, nil}
 }
 
@@ -209,12 +245,12 @@ func waitForPodRunning(
 	return api.PodUnknown, errors.New("timed out waiting for pod to start")
 }
 
-// limits takes a string representing CPU & memory limits,
+// limits takes a string representing CPU, memory and ephemeralStorage limits,
 // and returns a ResourceList with appropriately scaled Quantity
 // values for Kubernetes. This allows users to write "500m" for CPU,
-// and "50Mi" for memory (etc.)
-func limits(cpu, memory string) (api.ResourceList, error) {
-	var rCPU, rMem resource.Quantity
+// "50Mi" for memory and "1Gi" for ephemeral storage (etc.)
+func createResourceList(cpu, memory, ephemeralStorage string) (api.ResourceList, error) {
+	var rCPU, rMem, rStor resource.Quantity
 	var err error
 
 	parse := func(s string) (resource.Quantity, error) {
@@ -223,17 +259,25 @@ func limits(cpu, memory string) (api.ResourceList, error) {
 			return q, nil
 		}
 		if q, err = resource.ParseQuantity(s); err != nil {
-			return q, fmt.Errorf("parsing resource limit: %w", err)
+			return q, err
 		}
 		return q, nil
 	}
 
 	if rCPU, err = parse(cpu); err != nil {
-		return api.ResourceList{}, err
+		return api.ResourceList{}, &resourceQuantityError{resource: "cpu", value: cpu, inner: err}
 	}
 
 	if rMem, err = parse(memory); err != nil {
-		return api.ResourceList{}, err
+		return api.ResourceList{}, &resourceQuantityError{resource: "memory", value: memory, inner: err}
+	}
+
+	if rStor, err = parse(ephemeralStorage); err != nil {
+		return api.ResourceList{}, &resourceQuantityError{
+			resource: "ephemeralStorage",
+			value:    ephemeralStorage,
+			inner:    err,
+		}
 	}
 
 	l := make(api.ResourceList)
@@ -245,6 +289,9 @@ func limits(cpu, memory string) (api.ResourceList, error) {
 	if rMem != q {
 		l[api.ResourceMemory] = rMem
 	}
+	if rStor != q {
+		l[api.ResourceEphemeralStorage] = rStor
+	}
 
 	return l, nil
 }
@@ -261,3 +308,32 @@ func buildVariables(bv common.JobVariables) []api.EnvVar {
 	}
 	return e
 }
+
+// Sanitize labels to match Kubernetes restrictions from https://kubernetes.io/
+// /docs/concepts/overview/working-with-objects/labels/#syntax-and-character-set
+//nolint:gocognit
+func sanitizeLabel(value string) string {
+	mapFn := func(r rune) rune {
+		if r >= 'a' && r <= 'z' ||
+			r >= 'A' && r <= 'Z' ||
+			r >= '0' && r <= '9' ||
+			r == '-' || r == '_' || r == '.' {
+			return r
+		}
+		return '_'
+	}
+
+	// only alphanumerics, dashes (-), underscores (_), dots (.) are valid
+	value = strings.Map(mapFn, value)
+
+	// must start/end with alphanumerics only
+	value = strings.Trim(value, "-_.")
+
+	// length must be <= 63 characters
+	if len(value) > 63 {
+		value = value[:63]
+	}
+
+	// trim again if required after shortening
+	return strings.Trim(value, "-_.")
+}
