@@ -1,3 +1,6 @@
+//go:build integration
+// +build integration
+
 package shell_test
 
 import (
@@ -19,15 +22,41 @@ import (
 	"github.com/hashicorp/go-version"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+	"gitlab.com/gitlab-org/gitlab-runner/shells"
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 	"gitlab.com/gitlab-org/gitlab-runner/common/buildtest"
+	"gitlab.com/gitlab-org/gitlab-runner/executors/shell"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers"
+	"gitlab.com/gitlab-org/gitlab-runner/helpers/featureflags"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/test"
 	"gitlab.com/gitlab-org/gitlab-runner/session"
 	"gitlab.com/gitlab-org/gitlab-runner/shells/shellstest"
 )
 
+const integrationTestShellExecutor = "shell-integration-test"
+
+func TestMain(m *testing.M) {
+	code := 1
+	defer func() {
+		os.Exit(code)
+	}()
+
+	fmt.Println("Compiling gitlab-runner binary for tests")
+
+	targetDir, err := ioutil.TempDir("", "test_executor")
+	if err != nil {
+		panic("Error on preparing tmp directory for test executor binary")
+	}
+	defer os.RemoveAll(targetDir)
+
+	path := buildtest.MustBuildBinary("../..", filepath.Join(targetDir, "gitlab-runner-integration"))
+
+	shell.RegisterExecutor(integrationTestShellExecutor, path)
+
+	code = m.Run()
+}
+
 func gitInDir(dir string, args ...string) ([]byte, error) {
 	cmd := exec.Command("git", args...)
 	cmd.Dir = dir
@@ -35,7 +64,7 @@ func gitInDir(dir string, args ...string) ([]byte, error) {
 	return cmd.Output()
 }
 
-func skipOnGitWithMessage(t *testing.T, constraints string, message string) {
+func skipOnGit(t *testing.T, constraints string) {
 	out, err := gitInDir("", "version")
 	if err != nil {
 		t.Fatal("Can't detect git version", err)
@@ -66,22 +95,10 @@ func skipOnGitWithMessage(t *testing.T, constraints string, message string) {
 
 	shouldSkip := rules.Check(gitVersion)
 	if shouldSkip {
-		t.Skipf("Git %q found, skipping the test; %s", constraints, message)
+		t.Skipf("Git %q found, skipping the test", constraints)
 	}
 }
 
-func skipIfGitDoesNotSupportLFS(t *testing.T) {
-	skipOnGitWithMessage(t, "< 1.8.2", "available git version doesn't support LFS")
-}
-
-func skipOnGit(t *testing.T, constraints string) {
-	skipOnGitWithMessage(t, constraints, "")
-}
-
-func skipOnGit17x(t *testing.T) {
-	skipOnGit(t, "< 1.8")
-}
-
 func newBuild(t *testing.T, getBuildResponse common.JobResponse, shell string) (*common.Build, func()) {
 	dir, err := ioutil.TempDir("", "gitlab-runner-shell-executor-test")
 	if err != nil {
@@ -94,9 +111,11 @@ func newBuild(t *testing.T, getBuildResponse common.JobResponse, shell string) (
 		JobResponse: getBuildResponse,
 		Runner: &common.RunnerConfig{
 			RunnerSettings: common.RunnerSettings{
-				BuildsDir: dir,
-				Executor:  "shell",
-				Shell:     shell,
+				BuildsDir:           dir,
+				Executor:            integrationTestShellExecutor,
+				Shell:               shell,
+				GracefulKillTimeout: func(i int) *int { return &i }(5),
+				ForceKillTimeout:    func(i int) *int { return &i }(1),
 			},
 		},
 		SystemInterrupt: make(chan os.Signal, 1),
@@ -115,13 +134,17 @@ func newBuild(t *testing.T, getBuildResponse common.JobResponse, shell string) (
 
 func TestBuildSuccess(t *testing.T) {
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
-		successfulBuild, err := common.GetSuccessfulBuild()
-		assert.NoError(t, err)
-		build, cleanup := newBuild(t, successfulBuild, shell)
-		defer cleanup()
+		buildtest.WithEachFeatureFlag(t, func(t *testing.T, setup buildtest.BuildSetupFn) {
+			successfulBuild, err := common.GetSuccessfulBuild()
+			assert.NoError(t, err)
+			build, cleanup := newBuild(t, successfulBuild, shell)
+			defer cleanup()
 
-		err = buildtest.RunBuild(t, build)
-		assert.NoError(t, err)
+			setup(build)
+
+			err = buildtest.RunBuild(t, build)
+			assert.NoError(t, err)
+		}, featureflags.UsePowershellPathResolver)
 	})
 }
 
@@ -190,7 +213,8 @@ func TestMultistepBuild(t *testing.T) {
 
 				if tt.errExpected {
 					var buildErr *common.BuildError
-					assert.True(t, errors.As(err, &buildErr), "expected %T, got %T", buildErr, err)
+					assert.ErrorAs(t, err, &buildErr)
+					assert.Equal(t, 1, buildErr.ExitCode)
 					return
 				}
 				assert.NoError(t, err)
@@ -199,6 +223,57 @@ func TestMultistepBuild(t *testing.T) {
 	}
 }
 
+func TestBuildJobStatusEnvVars(t *testing.T) {
+	tests := map[string]struct {
+		fail   bool
+		assert func(t *testing.T, err error, build *common.Build, out string)
+	}{
+		"state on failure": {
+			fail: true,
+			assert: func(t *testing.T, err error, build *common.Build, out string) {
+				assert.Error(t, err)
+				assert.Contains(t, out, "CI_JOB_STATUS=failed")
+				assert.Equal(t, common.BuildRunRuntimeFailed, build.CurrentState())
+			},
+		},
+		"state on success": {
+			fail: false,
+			assert: func(t *testing.T, err error, build *common.Build, out string) {
+				assert.NoError(t, err)
+				assert.Contains(t, out, "CI_JOB_STATUS=success")
+				assert.Equal(t, common.BuildRunRuntimeSuccess, build.CurrentState())
+			},
+		},
+	}
+
+	expectedStages := []common.BuildStage{
+		common.BuildStagePrepare,
+		common.BuildStage("step_env"),
+		common.BuildStageAfterScript,
+	}
+
+	for tn, tc := range tests {
+		t.Run(tn, func(t *testing.T) {
+			shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+				multistepBuildScript, err := common.GetRemoteFailingMultistepBuildPrintVars(shell, tc.fail, "CI_JOB_STATUS")
+				require.NoError(t, err)
+
+				build, cleanup := newBuild(t, multistepBuildScript, shell)
+				defer cleanup()
+
+				out, err := buildtest.RunBuildReturningOutput(t, build)
+
+				assert.Contains(t, out, "CI_JOB_STATUS=running")
+				for _, stage := range expectedStages {
+					assert.Contains(t, out, common.GetStageDescription(stage))
+				}
+
+				tc.assert(t, err, build, out)
+			})
+		})
+	}
+}
+
 func TestRawVariableOutput(t *testing.T) {
 	tests := map[string]struct {
 		command string
@@ -239,42 +314,32 @@ func TestRawVariableOutput(t *testing.T) {
 	})
 }
 
-func TestBuildAbort(t *testing.T) {
+func TestBuildCancel(t *testing.T) {
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
-		longRunningBuild, err := common.GetLongRunningBuild()
-		assert.NoError(t, err)
-		build, cleanup := newBuild(t, longRunningBuild, shell)
+		build, cleanup := newBuild(t, common.JobResponse{}, shell)
 		defer cleanup()
 
-		abortTimer := time.AfterFunc(time.Second, func() {
-			t.Log("Interrupt")
-			build.SystemInterrupt <- os.Interrupt
-		})
-		defer abortTimer.Stop()
+		updateSleepForCMD := func(build *common.Build) {
+			if shell != "cmd" {
+				return
+			}
 
-		err = buildtest.RunBuild(t, build)
-		assert.EqualError(t, err, "aborted: interrupt")
+			resp, err := common.GetRemoteLongRunningBuildWithAfterScriptCMD()
+			require.NoError(t, err)
+
+			build.JobResponse = resp
+		}
+
+		buildtest.RunBuildWithCancel(t, build.Runner, updateSleepForCMD)
 	})
 }
 
-func TestBuildCancel(t *testing.T) {
+func TestBuildMasking(t *testing.T) {
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
-		longRunningBuild, err := common.GetLongRunningBuild()
-		assert.NoError(t, err)
-		build, cleanup := newBuild(t, longRunningBuild, shell)
+		build, cleanup := newBuild(t, common.JobResponse{}, shell)
 		defer cleanup()
 
-		trace := &common.Trace{Writer: os.Stdout}
-
-		cancelTimer := time.AfterFunc(time.Second, func() {
-			t.Log("Cancel")
-			trace.Cancel()
-		})
-		defer cancelTimer.Stop()
-
-		err = buildtest.RunBuildWithTrace(t, build, trace)
-		assert.EqualError(t, err, "canceled")
-		assert.IsType(t, &common.BuildError{}, err)
+		buildtest.RunBuildWithMasking(t, build.Runner, nil)
 	})
 }
 
@@ -340,6 +405,25 @@ func TestBuildWithHeadLock(t *testing.T) {
 	})
 }
 
+func TestBuildWithLeftoverConfigLock(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		successfulBuild, err := common.GetSuccessfulBuild()
+		assert.NoError(t, err)
+		build, cleanup := newBuild(t, successfulBuild, shell)
+		defer cleanup()
+
+		err = buildtest.RunBuild(t, build)
+		assert.NoError(t, err)
+
+		build.JobResponse.AllowGitFetch = true
+		err = ioutil.WriteFile(build.BuildDir+"/.git/config.lock", []byte{}, os.ModeSticky)
+		require.NoError(t, err)
+
+		err = buildtest.RunBuild(t, build)
+		assert.NoError(t, err)
+	})
+}
+
 func TestBuildWithGitLFSHook(t *testing.T) {
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetSuccessfulBuild()
@@ -397,6 +481,7 @@ func TestBuildWithGitStrategyNoneWithoutLFS(t *testing.T) {
 		defer cleanup()
 
 		build.Runner.PreCloneScript = "echo pre-clone-script"
+		build.Runner.PostCloneScript = "echo post-clone-script"
 		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "none"})
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
@@ -404,13 +489,12 @@ func TestBuildWithGitStrategyNoneWithoutLFS(t *testing.T) {
 		assert.NotContains(t, out, "pre-clone-script")
 		assert.NotContains(t, out, "Created fresh repository")
 		assert.NotContains(t, out, "Fetching changes")
+		assert.NotContains(t, out, "post-clone-script")
 		assert.Contains(t, out, "Skipping Git repository setup")
 	})
 }
 
 func TestBuildWithGitStrategyNoneWithLFS(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulLFSBuild()
 		assert.NoError(t, err)
@@ -436,6 +520,7 @@ func TestBuildWithGitStrategyFetchWithoutLFS(t *testing.T) {
 		defer cleanup()
 
 		build.Runner.PreCloneScript = "echo pre-clone-script"
+		build.Runner.PostCloneScript = "echo post-clone-script"
 		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"})
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
@@ -448,12 +533,11 @@ func TestBuildWithGitStrategyFetchWithoutLFS(t *testing.T) {
 		assert.Contains(t, out, "Fetching changes")
 		assert.Regexp(t, "Checking out [a-f0-9]+ as", out)
 		assert.Contains(t, out, "pre-clone-script")
+		assert.Contains(t, out, "post-clone-script")
 	})
 }
 
 func TestBuildWithGitStrategyFetchWithLFS(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulBuild()
 		assert.NoError(t, err)
@@ -479,8 +563,6 @@ func TestBuildWithGitStrategyFetchWithLFS(t *testing.T) {
 }
 
 func TestBuildWithGitStrategyFetchWithUserDisabledLFS(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulBuild()
 		assert.NoError(t, err)
@@ -521,6 +603,7 @@ func TestBuildWithGitStrategyFetchNoCheckoutWithoutLFS(t *testing.T) {
 		defer cleanup()
 
 		build.Runner.PreCloneScript = "echo pre-clone-script"
+		build.Runner.PostCloneScript = "echo post-clone-script"
 		build.Variables = append(
 			build.Variables,
 			common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"},
@@ -537,12 +620,11 @@ func TestBuildWithGitStrategyFetchNoCheckoutWithoutLFS(t *testing.T) {
 		assert.Contains(t, out, "Fetching changes")
 		assert.Contains(t, out, "Skipping Git checkout")
 		assert.Contains(t, out, "pre-clone-script")
+		assert.Contains(t, out, "post-clone-script")
 	})
 }
 
 func TestBuildWithGitStrategyFetchNoCheckoutWithLFS(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulLFSBuild()
 		assert.NoError(t, err)
@@ -579,6 +661,7 @@ func TestBuildWithGitStrategyCloneWithoutLFS(t *testing.T) {
 		defer cleanup()
 
 		build.Runner.PreCloneScript = "echo pre-clone-script"
+		build.Runner.PostCloneScript = "echo post-clone-script"
 		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "clone"})
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
@@ -590,12 +673,11 @@ func TestBuildWithGitStrategyCloneWithoutLFS(t *testing.T) {
 		assert.Contains(t, out, "Created fresh repository")
 		assert.Regexp(t, "Checking out [a-f0-9]+ as", out)
 		assert.Contains(t, out, "pre-clone-script")
+		assert.Contains(t, out, "post-clone-script")
 	})
 }
 
 func TestBuildWithGitStrategyCloneWithLFS(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulLFSBuild()
 		assert.NoError(t, err)
@@ -612,8 +694,6 @@ func TestBuildWithGitStrategyCloneWithLFS(t *testing.T) {
 }
 
 func TestBuildWithGitStrategyCloneWithUserDisabledLFS(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulLFSBuild()
 		assert.NoError(t, err)
@@ -641,6 +721,7 @@ func TestBuildWithGitStrategyCloneNoCheckoutWithoutLFS(t *testing.T) {
 		defer cleanup()
 
 		build.Runner.PreCloneScript = "echo pre-clone-script"
+		build.Runner.PostCloneScript = "echo post-clone-script"
 		build.Variables = append(
 			build.Variables,
 			common.JobVariable{Key: "GIT_STRATEGY", Value: "clone"},
@@ -656,12 +737,11 @@ func TestBuildWithGitStrategyCloneNoCheckoutWithoutLFS(t *testing.T) {
 		assert.Contains(t, out, "Created fresh repository")
 		assert.Contains(t, out, "Skipping Git checkout")
 		assert.Contains(t, out, "pre-clone-script")
+		assert.Contains(t, out, "post-clone-script")
 	})
 }
 
 func TestBuildWithGitStrategyCloneNoCheckoutWithLFS(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulLFSBuild()
 		assert.NoError(t, err)
@@ -683,8 +763,6 @@ func TestBuildWithGitStrategyCloneNoCheckoutWithLFS(t *testing.T) {
 }
 
 func TestBuildWithSubmoduleLFSPullsLFSObject(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulBuild()
 		assert.NoError(t, err)
@@ -708,8 +786,6 @@ func TestBuildWithSubmoduleLFSPullsLFSObject(t *testing.T) {
 }
 
 func TestBuildWithSubmoduleLFSDisabledSmudging(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulBuild()
 		assert.NoError(t, err)
@@ -763,6 +839,87 @@ func TestBuildWithGitSubmoduleStrategyNone(t *testing.T) {
 	}
 }
 
+func TestBuildWithGitSubmodulePaths(t *testing.T) {
+	// Some of these fail on earlier versions of git
+	// We can just skip it since we pass them directly to git and don't care for version support
+	skipOnGit(t, "< 1.9")
+
+	tests := map[string]struct {
+		paths                   string
+		expectedBuildError      error
+		expectedSubmoduleExists bool
+	}{
+		"include submodule": {
+			paths:                   "gitlab-grack",
+			expectedBuildError:      nil,
+			expectedSubmoduleExists: true,
+		},
+		"exclude submodule": {
+			paths:                   ":(exclude)gitlab-grack",
+			expectedBuildError:      nil,
+			expectedSubmoduleExists: false,
+		},
+		"exclude submodule with single space": {
+			paths:                   ":(exclude) gitlab-grack",
+			expectedBuildError:      nil,
+			expectedSubmoduleExists: true,
+		},
+		"exclude submodule with multiple spaces": {
+			paths:                   ":(exclude)  gitlab-grack",
+			expectedBuildError:      nil,
+			expectedSubmoduleExists: true,
+		},
+		"exclude submodule with space between all statements": {
+			paths:                   ": (exclude) gitlab-grack",
+			expectedBuildError:      &exec.ExitError{},
+			expectedSubmoduleExists: false,
+		},
+		"exclude submodule invalid": {
+			paths:                   "::::(exclude) gitlab-grack",
+			expectedBuildError:      &exec.ExitError{},
+			expectedSubmoduleExists: false,
+		},
+		"empty": {
+			paths:                   "    ",
+			expectedBuildError:      nil,
+			expectedSubmoduleExists: true,
+		},
+	}
+
+	for tn, tt := range tests {
+		t.Run(tn, func(t *testing.T) {
+			shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+				successfulBuild, err := common.GetSuccessfulBuild()
+				assert.NoError(t, err)
+
+				build, cleanup := newBuild(t, successfulBuild, shell)
+				defer cleanup()
+
+				build.Variables = append(
+					build.Variables,
+					common.JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "normal"},
+					common.JobVariable{Key: "GIT_SUBMODULE_PATHS", Value: tt.paths},
+				)
+
+				out, err := buildtest.RunBuildReturningOutput(t, build)
+				err = errors.Unwrap(err)
+				require.IsType(t, err, tt.expectedBuildError)
+
+				assert.NotContains(t, out, "Skipping Git submodules setup")
+				assert.Contains(t, out, "Updating/initializing submodules...")
+
+				_, err = os.Stat(filepath.Join(build.BuildDir, "gitlab-grack", ".git"))
+				if tt.expectedSubmoduleExists {
+					require.NoError(t, err, "Submodule should have been initialized")
+					return
+				}
+
+				require.True(t, errors.Is(err, os.ErrNotExist), "Submodule is initialized but shouldn't be")
+			})
+		})
+	}
+}
+
 func TestBuildWithGitSubmoduleStrategyNormal(t *testing.T) {
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetSuccessfulBuild()
@@ -787,8 +944,6 @@ func TestBuildWithGitSubmoduleStrategyNormal(t *testing.T) {
 }
 
 func TestBuildWithGitSubmoduleStrategyRecursive(t *testing.T) {
-	skipOnGit17x(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetSuccessfulBuild()
 		assert.NoError(t, err)
@@ -812,8 +967,6 @@ func TestBuildWithGitSubmoduleStrategyRecursive(t *testing.T) {
 }
 
 func TestBuildWithGitFetchSubmoduleStrategyRecursive(t *testing.T) {
-	skipOnGit17x(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetSuccessfulBuild()
 		assert.NoError(t, err)
@@ -851,6 +1004,99 @@ func TestBuildWithGitFetchSubmoduleStrategyRecursive(t *testing.T) {
 	})
 }
 
+func TestBuildGitCloneStrategyCleanup(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		successfulBuild, err := common.GetSuccessfulBuild()
+		assert.NoError(t, err)
+		build, cleanup := newBuild(t, successfulBuild, shell)
+		defer cleanup()
+		build.Runner.RunnerSettings.Shell = shell
+
+		buildtest.RunBuildWithCleanupGitClone(t, build)
+
+		_, err = os.Stat(build.FullProjectDir())
+		assert.Error(t, err, "cleanup should have removed the entire build directory")
+	})
+}
+
+func TestBuildGitFetchStrategyCleanup(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		untrackedFilename := "untracked"
+		successfulBuild, err := common.GetLocalBuildResponse(
+			buildtest.GetNewUntrackedFileIntoSubmodulesCommands(untrackedFilename, "", "")...,
+		)
+
+		assert.NoError(t, err)
+		build, cleanup := newBuild(t, successfulBuild, shell)
+		defer cleanup()
+		build.Runner.RunnerSettings.Shell = shell
+
+		buildtest.RunBuildWithCleanupGitFetch(t, build, untrackedFilename)
+
+		_, err = os.Stat(build.BuildDir)
+		assert.NoError(t, err, "cleanup should not delete entire build directory")
+		_, err = os.Stat(filepath.Join(build.BuildDir, untrackedFilename))
+		assert.Error(t, err, "cleanup should have removed the untracked file")
+	})
+}
+
+func TestBuildGitFetchStrategySubmoduleNormalCleanup(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		untrackedFilename, untrackedFileInSubmodule := "untracked", "untracked_in_submodule"
+		successfulBuild, err := common.GetLocalBuildResponse(
+			buildtest.GetNewUntrackedFileIntoSubmodulesCommands(untrackedFilename, untrackedFileInSubmodule, "")...,
+		)
+
+		assert.NoError(t, err)
+		build, cleanup := newBuild(t, successfulBuild, shell)
+		defer cleanup()
+
+		buildtest.RunBuildWithCleanupNormalSubmoduleStrategy(t, build, untrackedFilename, untrackedFileInSubmodule)
+
+		_, err = os.Stat(build.BuildDir)
+		assert.NoError(t, err, "cleanup should not delete entire build directory")
+		_, err = os.Stat(filepath.Join(build.BuildDir, untrackedFilename))
+		assert.Error(t, err, "cleanup should have removed untracked file in main repository")
+		_, err = os.Stat(filepath.Join(build.BuildDir, "gitlab-grack", untrackedFileInSubmodule))
+		assert.Error(t, err, "cleanup should have removed untracked file in submodule")
+	})
+}
+
+func TestBuildGitFetchStrategySubmoduleRecursiveCleanup(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		untrackedFile := "untracked_file"
+		untrackedFileInSubmodule := "untracked_file_in_submodule"
+		untrackedFileInSubSubmodule := "untracked_file_in_sub_submodule"
+
+		successfulBuild, err := common.GetLocalBuildResponse(
+			buildtest.GetNewUntrackedFileIntoSubmodulesCommands(untrackedFile, untrackedFileInSubmodule, untrackedFileInSubSubmodule)...,
+		)
+
+		assert.NoError(t, err)
+		build, cleanup := newBuild(t, successfulBuild, shell)
+		defer cleanup()
+
+		buildtest.RunBuildWithCleanupRecursiveSubmoduleStrategy(
+			t,
+			build,
+			untrackedFile,
+			untrackedFileInSubmodule,
+			untrackedFileInSubSubmodule,
+		)
+
+		_, err = os.Stat(build.BuildDir)
+		assert.NoError(t, err, "cleanup should not delete entire build directory")
+		_, err = os.Stat(filepath.Join(build.BuildDir, untrackedFile))
+		assert.Error(t, err, "cleanup should have removed untracked file in main repository")
+
+		_, err = os.Stat(filepath.Join(build.BuildDir, "gitlab-grack", untrackedFileInSubmodule))
+		assert.Error(t, err, "cleanup should have removed untracked file in submodule")
+
+		_, err = os.Stat(filepath.Join(build.BuildDir, "gitlab-grack", "tests", "example", untrackedFileInSubSubmodule))
+		assert.Error(t, err, "cleanup should have removed untracked file in submodule's submodule")
+	})
+}
+
 func TestBuildWithGitSubmoduleStrategyInvalid(t *testing.T) {
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetSuccessfulBuild()
@@ -990,7 +1236,7 @@ func TestBuildMultilineCommand(t *testing.T) {
 }
 
 func TestBuildWithBrokenGitSSLCAInfo(t *testing.T) {
-	skipOnGit17x(t)
+	skipOnGit(t, "< 1.9")
 	skipOnGit(t, ">= 2.10.2")
 
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
@@ -1033,6 +1279,7 @@ func TestBuildWithGitSSLAndStrategyFetch(t *testing.T) {
 		defer cleanup()
 
 		build.Runner.PreCloneScript = "echo pre-clone-script"
+		build.Runner.PostCloneScript = "echo post-clone-script"
 		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"})
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
@@ -1045,6 +1292,7 @@ func TestBuildWithGitSSLAndStrategyFetch(t *testing.T) {
 		assert.Contains(t, out, "Fetching changes")
 		assert.Regexp(t, "Checking out [a-f0-9]+ as", out)
 		assert.Contains(t, out, "pre-clone-script")
+		assert.Contains(t, out, "post-clone-script")
 	})
 }
 
@@ -1085,7 +1333,7 @@ func TestBuildChangesBranchesWhenFetchingRepo(t *testing.T) {
 		build.GitInfo = common.GetLFSGitInfo(build.GitInfo.RepoURL)
 		out, err = buildtest.RunBuildReturningOutput(t, build)
 		assert.NoError(t, err)
-		assert.Contains(t, out, "Checking out 2371dd05 as add-lfs-object...")
+		assert.Contains(t, out, "Checking out c8f2a61d as add-lfs-object...")
 	})
 }
 
@@ -1165,9 +1413,7 @@ func TestInteractiveTerminal(t *testing.T) {
 
 	for _, c := range cases {
 		t.Run(c.shell, func(t *testing.T) {
-			if helpers.SkipIntegrationTests(t, c.app) {
-				t.Skip()
-			}
+			helpers.SkipIntegrationTests(t, c.app)
 
 			successfulBuild, err := common.GetLocalBuildResponse(c.command)
 			require.NoError(t, err)
@@ -1193,11 +1439,11 @@ func TestInteractiveTerminal(t *testing.T) {
 			}()
 
 			// Wait until the build starts.
-			for build.Session.Mux() == nil {
+			for build.Session.Handler() == nil {
 				time.Sleep(10 * time.Millisecond)
 			}
 
-			srv := httptest.NewServer(build.Session.Mux())
+			srv := httptest.NewServer(build.Session.Handler())
 			defer srv.Close()
 
 			u := url.URL{
@@ -1274,3 +1520,177 @@ func TestBuildWithGitCleanFlags(t *testing.T) {
 		assert.Error(t, err, "cleanup_file does not exist")
 	})
 }
+
+func TestSanitizeGitDirectory(t *testing.T) {
+	test.SkipIfGitLabCIOn(t, test.OSWindows)
+
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		jobResponse, err := common.GetLocalBuildResponse(
+			"git remote set-url origin /tmp/some/invalid/directory",
+		)
+
+		build, cleanup := newBuild(t, jobResponse, shell)
+		defer cleanup()
+
+		build.Variables = append(
+			build.Variables,
+			common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"},
+			common.JobVariable{Key: featureflags.EnableJobCleanup, Value: "true"},
+		)
+
+		err = buildtest.RunBuild(t, build)
+		require.NoError(t, err)
+
+		_, err = os.Stat(filepath.Join(build.FullProjectDir(), ".git", "config"))
+		assert.True(t, errors.Is(err, os.ErrNotExist))
+
+		out, err := gitInDir(build.BuildDir, "init", "--template", filepath.Join(build.BuildDir, "git-template"))
+		assert.NoError(t, err)
+		assert.Contains(t, string(out), "Reinitialized existing Git repository")
+
+		_, err = gitInDir(build.BuildDir, "fsck")
+		assert.NoError(t, err)
+	})
+}
+
+func TestBuildFileVariablesRemoval(t *testing.T) {
+	getJobResponse := func(t *testing.T, jobResponseRequester func() (common.JobResponse, error)) common.JobResponse {
+		jobResponse, err := jobResponseRequester()
+		require.NoError(t, err)
+
+		return jobResponse
+	}
+
+	tests := map[string]struct {
+		jobResponse common.JobResponse
+	}{
+		"succeeded job": {
+			jobResponse: getJobResponse(t, common.GetSuccessfulBuild),
+		},
+		"failed job": {
+			jobResponse: getJobResponse(t, common.GetFailedBuild),
+		},
+	}
+
+	for tn, tt := range tests {
+		t.Run(tn, func(t *testing.T) {
+			shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+				buildtest.WithEachFeatureFlag(t, func(t *testing.T, setup buildtest.BuildSetupFn) {
+					build, cleanup := newBuild(t, tt.jobResponse, shell)
+					defer cleanup()
+
+					testVariableName := "TEST_VARIABLE"
+
+					build.Variables = append(
+						build.Variables,
+						common.JobVariable{Key: testVariableName, Value: "test", File: true},
+					)
+
+					setup(build)
+
+					_ = buildtest.RunBuild(t, build)
+
+					tmpDir := fmt.Sprintf("%s.tmp", build.BuildDir)
+					variableFile := filepath.Join(tmpDir, testVariableName)
+
+					_, err := os.Stat(variableFile)
+					assert.Error(t, err)
+					assert.ErrorIs(t, err, os.ErrNotExist)
+				}, featureflags.UsePowershellPathResolver)
+			})
+		})
+	}
+}
+
+func TestBuildLogLimitExceeded(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		build, cleanup := newBuild(t, common.JobResponse{}, shell)
+		defer cleanup()
+
+		buildtest.RunBuildWithJobOutputLimitExceeded(t, build.Runner, nil)
+	})
+}
+
+func TestBuildInvokeBinaryHelper(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		buildtest.WithEachFeatureFlag(t, func(t *testing.T, setup buildtest.BuildSetupFn) {
+			successfulBuild, err := common.GetRemoteSuccessfulBuild()
+			require.NoError(t, err)
+
+			build, cleanup := newBuild(t, successfulBuild, shell)
+			defer cleanup()
+
+			setup(build)
+
+			dir, err := ioutil.TempDir("", "")
+			require.NoError(t, err)
+			defer os.RemoveAll(dir)
+
+			build.Runner.RunnerSettings.BuildsDir = filepath.Join(dir, "build")
+			build.Runner.RunnerSettings.CacheDir = filepath.Join(dir, "cache")
+
+			build.Cache = append(build.Cache, common.Cache{
+				Key:    "cache",
+				Paths:  []string{"*"},
+				Policy: common.CachePolicyPullPush,
+			})
+
+			out, err := buildtest.RunBuildReturningOutput(t, build)
+			assert.NoError(t, err)
+			assert.NotContains(t, out, "Extracting cache is disabled.")
+			assert.NotContains(t, out, "Creating cache is disabled.")
+			assert.Contains(t, out, "Created cache")
+		}, featureflags.UsePowershellPathResolver)
+	})
+}
+
+func TestBuildPwshHandlesSyntaxErrors(t *testing.T) {
+	helpers.SkipIntegrationTests(t, shells.SNPwsh)
+
+	successfulBuild, err := common.GetLocalBuildResponse("some syntax error\nWrite-Output $PSVersionTable")
+	require.NoError(t, err)
+
+	build, cleanup := newBuild(t, successfulBuild, shells.SNPwsh)
+	defer cleanup()
+
+	out, err := buildtest.RunBuildReturningOutput(t, build)
+	assert.Error(t, err)
+	assert.NotContains(t, out, "PSEdition")
+}
+
+func TestBuildPwshHandlesScriptEncodingCorrectly(t *testing.T) {
+	helpers.SkipIntegrationTests(t, shells.SNPwsh)
+
+	successfulBuild, err := common.GetLocalBuildResponse("echo $Env:GL_Test1 | Format-Hex")
+	require.NoError(t, err)
+
+	build, cleanup := newBuild(t, successfulBuild, shells.SNPwsh)
+	defer cleanup()
+
+	build.Variables = append(build.Variables, common.JobVariable{
+		Key:   "GL_Test1",
+		Value: "∅",
+		Raw:   true,
+	})
+
+	out, err := buildtest.RunBuildReturningOutput(t, build)
+	assert.NoError(t, err)
+	assert.Contains(t, out, "E2 88 85")
+}
+
+func TestBuildScriptSections(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		if shell == "cmd" || shell == "pwsh" || shell == "powershell" {
+			// support for pwsh and powershell tracked in https://gitlab.com/gitlab-org/gitlab-runner/-/issues/28119
+			t.Skip("CMD not supported")
+		}
+		build, cleanup := newBuild(t, common.JobResponse{}, shell)
+		defer cleanup()
+
+		successfulBuild, err := common.GetSuccessfulBuild()
+		require.NoError(t, err)
+		build.JobResponse = successfulBuild
+		build.Runner.RunnerSettings.Shell = shell
+		buildtest.RunBuildWithSections(t, build)
+	})
+}
