@@ -1,4 +1,5 @@
-// +build linux darwin freebsd openbsd
+//go:build aix || android || darwin || dragonfly || freebsd || hurd || illumos || linux || netbsd || openbsd || solaris
+// +build aix android darwin dragonfly freebsd hurd illumos linux netbsd openbsd solaris
 
 package archives
 
@@ -31,7 +32,7 @@ func createZipUIDGidField(w io.Writer, fi os.FileInfo) (err error) {
 	if err == nil {
 		err = binary.Write(w, binary.LittleEndian, &ugField)
 	}
-	return nil
+	return err
 }
 
 func processZipUIDGidField(data []byte, file *zip.FileHeader) error {
