@@ -1,4 +1,4 @@
-package docker_helpers
+package docker
 
 import (
 	"time"
@@ -13,5 +13,5 @@ type Machine interface {
 	Exist(name string) bool
 
 	CanConnect(name string, skipCache bool) bool
-	Credentials(name string) (DockerCredentials, error)
+	Credentials(name string) (Credentials, error)
 }
