@@ -1,3 +1,4 @@
+//go:build windows
 // +build windows
 
 // This implementation only works when compiled for Windows
@@ -7,21 +8,34 @@ package path
 
 import (
 	"path/filepath"
+	"regexp"
 	"strings"
 )
 
-type windowsPath struct{}
+type windowsPath struct {
+}
+
+// windowsNamedPipesExp matches a named pipe path (starts with `\\.\pipe\`, possibly with / instead of \)
+var windowsNamedPipe = regexp.MustCompile(`(?i)^[/\\]{2}\.[/\\]pipe[/\\][^:*?"<>|\r\n]+$`)
 
 func (p *windowsPath) Join(elem ...string) string {
 	return filepath.Join(elem...)
 }
 
 func (p *windowsPath) IsAbs(path string) bool {
+	if windowsNamedPipe.MatchString(path) {
+		return true
+	}
+
 	path = filepath.Clean(path)
 	return filepath.IsAbs(path)
 }
 
 func (p *windowsPath) IsRoot(path string) bool {
+	if windowsNamedPipe.MatchString(path) {
+		return false
+	}
+
 	path = filepath.Clean(path)
 	return filepath.IsAbs(path) && filepath.Dir(path) == path
 }
@@ -42,6 +56,7 @@ func (p *windowsPath) Contains(basePath, targetPath string) bool {
 	return true
 }
 
-func NewWindowsPath() Path {
+//revive:disable:unexported-return
+func NewWindowsPath() *windowsPath {
 	return &windowsPath{}
 }
