@@ -1,3 +1,6 @@
+//go:build !integration
+// +build !integration
+
 package prometheus
 
 import (
@@ -10,7 +13,7 @@ import (
 
 func callFireConcurrent(lh *LogHook, repeats int, finish chan bool) {
 	for i := 0; i < repeats; i++ {
-		lh.Fire(&logrus.Entry{
+		_ = lh.Fire(&logrus.Entry{
 			Level: logrus.ErrorLevel,
 		})
 		finish <- true
