@@ -5,15 +5,18 @@ import (
 	"fmt"
 	"os"
 	"runtime"
+	"time"
 
-	"github.com/getsentry/raven-go"
+	sentrygo "github.com/getsentry/sentry-go"
 	"github.com/sirupsen/logrus"
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 )
 
+const messageFlushTimeout = 10 * time.Second
+
 type LogHook struct {
-	client *raven.Client
+	hub *sentrygo.Hub
 }
 
 func (s *LogHook) Levels() []logrus.Level {
@@ -24,8 +27,15 @@ func (s *LogHook) Levels() []logrus.Level {
 	}
 }
 
+func sentryLevelFromLogrusLevel(logrusLevel logrus.Level) sentrygo.Level {
+	if logrusLevel == logrus.PanicLevel || logrusLevel == logrus.FatalLevel {
+		return sentrygo.LevelFatal
+	}
+	return sentrygo.LevelError
+}
+
 func (s *LogHook) Fire(entry *logrus.Entry) error {
-	if s.client == nil {
+	if s.hub == nil {
 		return nil
 	}
 
@@ -33,17 +43,19 @@ func (s *LogHook) Fire(entry *logrus.Entry) error {
 	for key, value := range entry.Data {
 		tags[key] = fmt.Sprint(value)
 	}
+	level := sentryLevelFromLogrusLevel(entry.Level)
 
-	switch entry.Level {
-	case logrus.PanicLevel:
-		s.client.CaptureErrorAndWait(errors.New(entry.Message), tags)
+	scope := s.hub.PushScope()
+	defer s.hub.PopScope()
 
-	case logrus.FatalLevel:
-		s.client.CaptureErrorAndWait(errors.New(entry.Message), tags)
+	scope.SetTags(tags)
+	scope.SetLevel(level)
 
-	case logrus.ErrorLevel:
-		s.client.CaptureError(errors.New(entry.Message), tags)
+	s.hub.CaptureException(errors.New(entry.Message))
+	if level == sentrygo.LevelFatal {
+		s.hub.Flush(messageFlushTimeout)
 	}
+
 	return nil
 }
 
@@ -57,10 +69,21 @@ func NewLogHook(dsn string) (lh LogHook, err error) {
 	tags["go-os"] = runtime.GOOS
 	tags["go-arch"] = runtime.GOARCH
 	tags["hostname"], _ = os.Hostname()
-	client, err := raven.NewWithTags(dsn, tags)
+
+	scope := sentrygo.NewScope()
+	client, err := sentrygo.NewClient(sentrygo.ClientOptions{
+		Dsn: dsn,
+	})
+
 	if err != nil {
 		return
 	}
-	lh.client = client
+
+	hub := sentrygo.NewHub(client, scope)
+	hub.ConfigureScope(func(scope *sentrygo.Scope) {
+		scope.SetTags(tags)
+	})
+	lh.hub = hub
+
 	return
 }
