@@ -2,13 +2,13 @@ package network
 
 import (
 	"bytes"
+	"context"
 	"fmt"
 	"io"
 	"io/ioutil"
 	"mime/multipart"
 	"net/http"
 	"net/url"
-	"os"
 	"runtime"
 	"strconv"
 	"sync"
@@ -130,6 +130,14 @@ func (n *GitLabClient) getLastUpdate(credentials requestCredentials) (lu string)
 	return cli.getLastUpdate()
 }
 
+// getFeatures enables features that are properties of networking client
+func (n *GitLabClient) getFeatures(features *common.FeaturesInfo) {
+	features.TraceReset = true
+	features.TraceChecksum = true
+	features.TraceSize = true
+	features.Cancelable = true
+}
+
 func (n *GitLabClient) getRunnerVersion(config common.RunnerConfig) common.VersionInfo {
 	info := common.VersionInfo{
 		Name:         common.NAME,
@@ -141,12 +149,16 @@ func (n *GitLabClient) getRunnerVersion(config common.RunnerConfig) common.Versi
 		Shell:        config.Shell,
 	}
 
+	n.getFeatures(&info.Features)
+
 	if executorProvider := common.GetExecutorProvider(config.Executor); executorProvider != nil {
 		_ = executorProvider.GetFeatures(&info.Features)
 
 		if info.Shell == "" {
 			info.Shell = executorProvider.GetDefaultShell()
 		}
+
+		executorProvider.GetConfigInfo(&config, &info.Config)
 	}
 
 	if shell := common.GetShell(info.Shell); shell != nil {
@@ -157,6 +169,7 @@ func (n *GitLabClient) getRunnerVersion(config common.RunnerConfig) common.Versi
 }
 
 func (n *GitLabClient) doRaw(
+	ctx context.Context,
 	credentials requestCredentials,
 	method, uri string,
 	request io.Reader,
@@ -168,10 +181,11 @@ func (n *GitLabClient) doRaw(
 		return nil, err
 	}
 
-	return c.do(uri, method, request, requestType, headers)
+	return c.do(ctx, uri, method, request, requestType, headers)
 }
 
 func (n *GitLabClient) doJSON(
+	ctx context.Context,
 	credentials requestCredentials,
 	method, uri string,
 	statusCode int,
@@ -183,7 +197,7 @@ func (n *GitLabClient) doJSON(
 		return clientError, err.Error(), nil
 	}
 
-	return c.doJSON(uri, method, statusCode, request, response)
+	return c.doJSON(ctx, uri, method, statusCode, request, response)
 }
 
 func (n *GitLabClient) getResponseTLSData(
@@ -211,6 +225,7 @@ func (n *GitLabClient) RegisterRunner(
 
 	var response common.RegisterRunnerResponse
 	result, statusText, resp := n.doJSON(
+		context.Background(),
 		&runner,
 		http.MethodPost,
 		"runners",
@@ -243,7 +258,15 @@ func (n *GitLabClient) VerifyRunner(runner common.RunnerCredentials) bool {
 		Token: runner.Token,
 	}
 
-	result, statusText, resp := n.doJSON(&runner, http.MethodPost, "runners/verify", http.StatusOK, &request, nil)
+	result, statusText, resp := n.doJSON(
+		context.Background(),
+		&runner,
+		http.MethodPost,
+		"runners/verify",
+		http.StatusOK,
+		&request,
+		nil,
+	)
 	if resp != nil {
 		defer func() { _ = resp.Body.Close() }()
 	}
@@ -270,7 +293,15 @@ func (n *GitLabClient) UnregisterRunner(runner common.RunnerCredentials) bool {
 		Token: runner.Token,
 	}
 
-	result, statusText, resp := n.doJSON(&runner, http.MethodDelete, "runners", http.StatusNoContent, &request, nil)
+	result, statusText, resp := n.doJSON(
+		context.Background(),
+		&runner,
+		http.MethodDelete,
+		"runners",
+		http.StatusNoContent,
+		&request,
+		nil,
+	)
 	if resp != nil {
 		defer func() { _ = resp.Body.Close() }()
 	}
@@ -310,6 +341,7 @@ func addTLSData(response *common.JobResponse, tlsData ResponseTLSData) {
 }
 
 func (n *GitLabClient) RequestJob(
+	ctx context.Context,
 	config common.RunnerConfig,
 	sessionInfo *common.SessionInfo,
 ) (*common.JobResponse, bool) {
@@ -322,6 +354,7 @@ func (n *GitLabClient) RequestJob(
 
 	var response common.JobResponse
 	result, statusText, httpResponse := n.doJSON(
+		ctx,
 		&config.RunnerCredentials,
 		http.MethodPost,
 		"jobs/request",
@@ -366,15 +399,19 @@ func (n *GitLabClient) UpdateJob(
 	config common.RunnerConfig,
 	jobCredentials *common.JobCredentials,
 	jobInfo common.UpdateJobInfo,
-) common.UpdateState {
+) common.UpdateJobResult {
 	request := common.UpdateJobRequest{
 		Info:          n.getRunnerVersion(config),
 		Token:         jobCredentials.Token,
 		State:         jobInfo.State,
 		FailureReason: jobInfo.FailureReason,
+		Checksum:      jobInfo.Output.Checksum, // deprecated
+		Output:        jobInfo.Output,
+		ExitCode:      jobInfo.ExitCode,
 	}
 
-	result, statusText, response := n.doJSON(
+	statusCode, statusText, response := n.doJSON(
+		context.Background(),
 		&config.RunnerCredentials,
 		http.MethodPut,
 		fmt.Sprintf("jobs/%d", jobInfo.ID),
@@ -382,35 +419,60 @@ func (n *GitLabClient) UpdateJob(
 		&request,
 		nil,
 	)
-	n.requestsStatusesMap.Append(config.RunnerCredentials.ShortDescription(), APIEndpointUpdateJob, result)
+	n.requestsStatusesMap.Append(config.RunnerCredentials.ShortDescription(), APIEndpointUpdateJob, statusCode)
 
-	remoteJobStateResponse := NewRemoteJobStateResponse(response)
-	log := config.Log().WithFields(logrus.Fields{
-		"code":       result,
-		"job":        jobInfo.ID,
-		"job-status": remoteJobStateResponse.RemoteState,
+	log := config.Log().WithField("job", jobInfo.ID)
+
+	return n.createUpdateJobResult(log, statusCode, statusText, response)
+}
+
+func (n *GitLabClient) createUpdateJobResult(
+	log *logrus.Entry,
+	statusCode int,
+	statusText string,
+	response *http.Response,
+) common.UpdateJobResult {
+	remoteJobStateResponse := NewRemoteJobStateResponse(response, log)
+
+	result := common.UpdateJobResult{
+		NewUpdateInterval: remoteJobStateResponse.RemoteUpdateInterval,
+		CancelRequested:   remoteJobStateResponse.IsCanceled(),
+	}
+
+	log = log.WithFields(logrus.Fields{
+		"code":            statusCode,
+		"job-status":      remoteJobStateResponse.RemoteState,
+		"update-interval": remoteJobStateResponse.RemoteUpdateInterval,
 	})
 
 	switch {
-	case remoteJobStateResponse.IsAborted():
-		log.Warningln("Submitting job to coordinator...", "aborted")
-		return common.UpdateAbort
-	case result == http.StatusOK:
+	case remoteJobStateResponse.IsFailed():
+		log.Warningln("Submitting job to coordinator...", "job failed")
+		result.State = common.UpdateAbort
+	case statusCode == http.StatusOK:
 		log.Debugln("Submitting job to coordinator...", "ok")
-		return common.UpdateSucceeded
-	case result == http.StatusNotFound:
-		log.Warningln("Submitting job to coordinator...", "aborted")
-		return common.UpdateAbort
-	case result == http.StatusForbidden:
+		result.State = common.UpdateSucceeded
+	case statusCode == http.StatusAccepted:
+		log.Debugln("Submitting job to coordinator...", "accepted, but not yet completed")
+		result.State = common.UpdateAcceptedButNotCompleted
+	case statusCode == http.StatusPreconditionFailed:
+		log.Debugln("Submitting job to coordinator...", "trace validation failed")
+		result.State = common.UpdateTraceValidationFailed
+	case statusCode == http.StatusNotFound:
+		log.Warningln("Submitting job to coordinator...", "not found")
+		result.State = common.UpdateAbort
+	case statusCode == http.StatusForbidden:
 		log.WithField("status", statusText).Errorln("Submitting job to coordinator...", "forbidden")
-		return common.UpdateAbort
-	case result == clientError:
+		result.State = common.UpdateAbort
+	case statusCode == clientError:
 		log.WithField("status", statusText).Errorln("Submitting job to coordinator...", "error")
-		return common.UpdateAbort
+		result.State = common.UpdateAbort
 	default:
 		log.WithField("status", statusText).Warningln("Submitting job to coordinator...", "failed")
-		return common.UpdateFailed
+		result.State = common.UpdateFailed
 	}
+
+	return result
 }
 
 func (n *GitLabClient) PatchTrace(
@@ -424,7 +486,7 @@ func (n *GitLabClient) PatchTrace(
 	baseLog := config.Log().WithField("job", id)
 	if len(content) == 0 {
 		baseLog.Debugln("Appending trace to coordinator...", "skipped due to empty patch")
-		return common.NewPatchTraceResult(startOffset, common.UpdateSucceeded, 0)
+		return common.NewPatchTraceResult(startOffset, common.PatchSucceeded, 0)
 	}
 
 	endOffset := startOffset + len(content)
@@ -437,10 +499,18 @@ func (n *GitLabClient) PatchTrace(
 	uri := fmt.Sprintf("jobs/%d/trace", id)
 	request := bytes.NewReader(content)
 
-	response, err := n.doRaw(&config.RunnerCredentials, "PATCH", uri, request, "text/plain", headers)
+	response, err := n.doRaw(
+		context.Background(),
+		&config.RunnerCredentials,
+		"PATCH",
+		uri,
+		request,
+		"text/plain",
+		headers,
+	)
 	if err != nil {
 		config.Log().Errorln("Appending trace to coordinator...", "error", err.Error())
-		return common.NewPatchTraceResult(startOffset, common.UpdateFailed, 0)
+		return common.NewPatchTraceResult(startOffset, common.PatchFailed, 0)
 	}
 
 	n.requestsStatusesMap.Append(
@@ -461,7 +531,7 @@ func (n *GitLabClient) PatchTrace(
 		"job-status":      tracePatchResponse.RemoteState,
 		"code":            response.StatusCode,
 		"status":          response.Status,
-		"update-interval": tracePatchResponse.RemoteTraceUpdateInterval,
+		"update-interval": tracePatchResponse.RemoteUpdateInterval,
 	})
 
 	return n.createPatchTraceResult(startOffset, tracePatchResponse, response, endOffset, log)
@@ -476,61 +546,75 @@ func (n *GitLabClient) createPatchTraceResult(
 ) common.PatchTraceResult {
 	result := common.PatchTraceResult{
 		SentOffset:        startOffset,
-		NewUpdateInterval: tracePatchResponse.RemoteTraceUpdateInterval,
+		NewUpdateInterval: tracePatchResponse.RemoteUpdateInterval,
+		CancelRequested:   tracePatchResponse.IsCanceled(),
 	}
 
 	switch {
-	case tracePatchResponse.IsAborted():
-		log.Warningln("Appending trace to coordinator...", "aborted")
-		result.State = common.UpdateAbort
+	case tracePatchResponse.IsFailed():
+		log.Warningln("Appending trace to coordinator...", "job failed")
+		result.State = common.PatchAbort
 
 		return result
 
 	case response.StatusCode == http.StatusAccepted:
 		log.Debugln("Appending trace to coordinator...", "ok")
 		result.SentOffset = endOffset
-		result.State = common.UpdateSucceeded
+		result.State = common.PatchSucceeded
 
 		return result
 
 	case response.StatusCode == http.StatusNotFound:
 		log.Warningln("Appending trace to coordinator...", "not-found")
-		result.State = common.UpdateNotFound
+		result.State = common.PatchNotFound
 
 		return result
 
 	case response.StatusCode == http.StatusRequestedRangeNotSatisfiable:
 		log.Warningln("Appending trace to coordinator...", "range mismatch")
 		result.SentOffset = tracePatchResponse.NewOffset()
-		result.State = common.UpdateRangeMismatch
+		result.State = common.PatchRangeMismatch
 
 		return result
 
 	case response.StatusCode == clientError:
 		log.Errorln("Appending trace to coordinator...", "error")
-		result.State = common.UpdateAbort
+		result.State = common.PatchAbort
 
 		return result
 
 	default:
 		log.Warningln("Appending trace to coordinator...", "failed")
-		result.State = common.UpdateFailed
+		result.State = common.PatchFailed
 
 		return result
 	}
 }
 
-func (n *GitLabClient) createArtifactsForm(mpw *multipart.Writer, reader io.Reader, baseName string) error {
-	wr, err := mpw.CreateFormFile("file", baseName)
-	if err != nil {
-		return err
-	}
+func (n *GitLabClient) createArtifactsForm(reader io.Reader, baseName string) (io.ReadCloser, string) {
+	pr, pw := io.Pipe()
 
-	_, err = io.Copy(wr, reader)
-	if err != nil {
-		return err
-	}
-	return nil
+	mpw := multipart.NewWriter(pw)
+
+	go func() {
+		defer func() {
+			_ = mpw.Close()
+			_ = pw.Close()
+		}()
+
+		wr, err := mpw.CreateFormFile("file", baseName)
+		if err != nil {
+			_ = pw.CloseWithError(err)
+			return
+		}
+
+		_, err = io.Copy(wr, reader)
+		if err != nil {
+			_ = pw.CloseWithError(err)
+		}
+	}()
+
+	return pr, mpw.FormDataContentType()
 }
 
 func uploadRawArtifactsQuery(options common.ArtifactsOptions) url.Values {
@@ -553,23 +637,16 @@ func uploadRawArtifactsQuery(options common.ArtifactsOptions) url.Values {
 
 func (n *GitLabClient) UploadRawArtifacts(
 	config common.JobCredentials,
-	reader io.Reader,
+	reader io.ReadCloser,
 	options common.ArtifactsOptions,
-) common.UploadState {
-	pr, pw := io.Pipe()
-	defer func() { _ = pr.Close() }()
+) (common.UploadState, string) {
+	defer func() {
+		_ = reader.Close()
+	}()
 
-	mpw := multipart.NewWriter(pw)
-
-	go func() {
-		defer func() {
-			_ = mpw.Close()
-			_ = pw.Close()
-		}()
-		err := n.createArtifactsForm(mpw, reader, options.BaseName)
-		if err != nil {
-			_ = pw.CloseWithError(err)
-		}
+	pr, contentType := n.createArtifactsForm(reader, options.BaseName)
+	defer func() {
+		_ = pr.Close()
 	}()
 
 	query := uploadRawArtifactsQuery(options)
@@ -577,11 +654,12 @@ func (n *GitLabClient) UploadRawArtifacts(
 	headers := make(http.Header)
 	headers.Set("JOB-TOKEN", config.Token)
 	res, err := n.doRaw(
+		context.Background(),
 		&config,
 		http.MethodPost,
 		fmt.Sprintf("jobs/%d/artifacts?%s", config.ID, query.Encode()),
 		pr,
-		mpw.FormDataContentType(),
+		contentType,
 		headers,
 	)
 
@@ -594,6 +672,9 @@ func (n *GitLabClient) UploadRawArtifacts(
 		log = log.WithField("responseStatus", res.Status)
 	}
 
+	closeWithLogging(log, pr, "pipe reader")
+	closeWithLogging(log, reader, "archive reader")
+
 	messagePrefix := "Uploading artifacts to coordinator..."
 	if options.Type != "" {
 		messagePrefix = fmt.Sprintf("Uploading artifacts as %q to coordinator...", options.Type)
@@ -601,43 +682,70 @@ func (n *GitLabClient) UploadRawArtifacts(
 
 	if err != nil {
 		log.WithError(err).Errorln(messagePrefix, "error")
-		return common.UploadFailed
+		return common.UploadFailed, ""
 	}
+
 	defer func() {
 		_, _ = io.Copy(ioutil.Discard, res.Body)
 		_ = res.Body.Close()
 	}()
 
-	return n.determineUploadState(res.StatusCode, log, messagePrefix)
+	return n.determineUploadState(res, log, messagePrefix)
+}
+
+func closeWithLogging(log logrus.FieldLogger, c io.Closer, name string) {
+	err := c.Close()
+	if err != nil {
+		log.WithError(err).Warningf("Error while closing the %s", name)
+	}
 }
 
 func (n *GitLabClient) determineUploadState(
-	statusCode int,
+	resp *http.Response,
 	log *logrus.Entry,
 	messagePrefix string,
-) common.UploadState {
-	switch statusCode {
+) (common.UploadState, string) {
+	statusText := getMessageFromJSONResponse(resp)
+
+	switch resp.StatusCode {
 	case http.StatusCreated:
-		log.Println(messagePrefix, "ok")
-		return common.UploadSucceeded
+		log.Println(messagePrefix, statusText)
+		return common.UploadSucceeded, ""
+	case http.StatusTemporaryRedirect:
+		return handleUploadRedirectionState(resp, log, messagePrefix, statusText)
 	case http.StatusForbidden:
-		log.WithField("status", statusCode).Errorln(messagePrefix, "forbidden")
-		return common.UploadForbidden
+		log.WithField("status", resp.StatusCode).Errorln(messagePrefix, statusText)
+		return common.UploadForbidden, ""
 	case http.StatusRequestEntityTooLarge:
-		log.WithField("status", statusCode).Errorln(messagePrefix, "too large archive")
-		return common.UploadTooLarge
+		log.WithField("status", resp.StatusCode).Errorln(messagePrefix, statusText)
+		return common.UploadTooLarge, ""
 	case http.StatusServiceUnavailable:
-		log.WithField("status", statusCode).Errorln(messagePrefix, "service unavailable")
-		return common.UploadServiceUnavailable
+		log.WithField("status", resp.StatusCode).Errorln(messagePrefix, statusText)
+		return common.UploadServiceUnavailable, ""
 	default:
-		log.WithField("status", statusCode).Warningln(messagePrefix, "failed")
-		return common.UploadFailed
+		log.WithField("status", resp.StatusCode).Warningln(messagePrefix, statusText)
+		return common.UploadFailed, ""
 	}
 }
 
+func handleUploadRedirectionState(
+	resp *http.Response,
+	log *logrus.Entry,
+	messagePrefix string,
+	statusText string,
+) (common.UploadState, string) {
+	location := resp.Header.Get("Location")
+	if location == "" {
+		log.WithField("status", resp.StatusCode).Errorln(messagePrefix, statusText, "empty location")
+		return common.UploadFailed, ""
+	}
+
+	return common.UploadRedirected, location
+}
+
 func (n *GitLabClient) DownloadArtifacts(
 	config common.JobCredentials,
-	artifactsFile string,
+	artifactsFile io.WriteCloser,
 	directDownload *bool,
 ) common.DownloadState {
 	query := url.Values{}
@@ -650,7 +758,7 @@ func (n *GitLabClient) DownloadArtifacts(
 	headers.Set("JOB-TOKEN", config.Token)
 	uri := fmt.Sprintf("jobs/%d/artifacts?%s", config.ID, query.Encode())
 
-	res, err := n.doRaw(&config, http.MethodGet, uri, nil, "", headers)
+	res, err := n.doRaw(context.Background(), &config, http.MethodGet, uri, nil, "", headers)
 
 	log := logrus.WithFields(logrus.Fields{
 		"id":    config.ID,
@@ -672,21 +780,14 @@ func (n *GitLabClient) DownloadArtifacts(
 
 	switch res.StatusCode {
 	case http.StatusOK:
-		file, err := os.Create(artifactsFile)
-		if err == nil {
-			defer func() { _ = file.Close() }()
-			_, err = io.Copy(file, res.Body)
-		}
-		if err != nil {
-			_ = file.Close()
-			_ = os.Remove(file.Name())
-			log.WithError(err).Errorln("Downloading artifacts from coordinator...", "error")
-			return common.DownloadFailed
-		}
-		log.Println("Downloading artifacts from coordinator...", "ok")
-		return common.DownloadSucceeded
+		return n.downloadArtifactFile(log, artifactsFile, res)
 	case http.StatusForbidden:
-		log.WithField("status", res.Status).Errorln("Downloading artifacts from coordinator...", "forbidden")
+		// We generally expect JSON responses from the GitLab API, but a
+		// 302 redirection to object storage may result in an XML
+		// response that might include important details why the request
+		// was rejected (e.g. Google VPC Service Controls).
+		statusText := getMessageFromJSONOrXMLResponse(res)
+		log.WithField("status", statusText).Errorln("Downloading artifacts from coordinator...", "forbidden")
 		return common.DownloadForbidden
 	case http.StatusNotFound:
 		log.Errorln("Downloading artifacts from coordinator...", "not found")
@@ -697,6 +798,25 @@ func (n *GitLabClient) DownloadArtifacts(
 	}
 }
 
+func (n *GitLabClient) downloadArtifactFile(
+	log logrus.FieldLogger,
+	file io.WriteCloser,
+	res *http.Response,
+) common.DownloadState {
+	_, err := io.Copy(file, res.Body)
+
+	closeWithLogging(log, file, "file writer")
+
+	if err != nil {
+		log.WithError(err).Errorln("Downloading artifacts from coordinator...", "error")
+		return common.DownloadFailed
+	}
+
+	log.Println("Downloading artifacts from coordinator...", "ok")
+
+	return common.DownloadSucceeded
+}
+
 func (n *GitLabClient) ProcessJob(
 	config common.RunnerConfig,
 	jobCredentials *common.JobCredentials,
