@@ -1,11 +1,14 @@
+//go:build !integration
+// +build !integration
+
 package session
 
 import (
+	"errors"
 	"net/http"
 	"net/http/httptest"
 	"testing"
 
-	"github.com/pkg/errors"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/mock"
 	"github.com/stretchr/testify/require"
@@ -42,7 +45,7 @@ func TestExecSuccessful(t *testing.T) {
 
 	w := httptest.NewRecorder()
 
-	session.Mux().ServeHTTP(w, req)
+	session.Handler().ServeHTTP(w, req)
 
 	resp := w.Result()
 	defer resp.Body.Close()
@@ -122,7 +125,7 @@ func TestExecFailedRequest(t *testing.T) {
 
 			w := httptest.NewRecorder()
 
-			session.Mux().ServeHTTP(w, req)
+			session.Handler().ServeHTTP(w, req)
 
 			resp := w.Result()
 			defer resp.Body.Close()
@@ -158,7 +161,7 @@ func TestDoNotAllowMultipleConnections(t *testing.T) {
 	req.Header.Add("Authorization", validToken)
 
 	w := httptest.NewRecorder()
-	session.Mux().ServeHTTP(w, req)
+	session.Handler().ServeHTTP(w, req)
 	resp := w.Result()
 	defer resp.Body.Close()
 	assert.Equal(t, http.StatusLocked, resp.StatusCode)
@@ -300,7 +303,7 @@ func TestProxy(t *testing.T) {
 				mockConn.On("ProxyRequest", mock.Anything, mock.Anything, mock.Anything, "80", mock.Anything).Once()
 			}
 
-			session.Mux().ServeHTTP(w, req)
+			session.Handler().ServeHTTP(w, req)
 
 			resp := w.Result()
 			defer resp.Body.Close()
@@ -309,6 +312,35 @@ func TestProxy(t *testing.T) {
 	}
 }
 
+func TestProxyVars(t *testing.T) {
+	tests := map[string]struct {
+		service      string
+		port         string
+		requestedURI string
+		ok           bool
+	}{
+		"empty":              {"", "", "", false},
+		"/":                  {"", "", "", false},
+		"//":                 {"", "", "", false},
+		"///":                {"", "", "/", false},
+		"/80/":               {"", "80", "", false},
+		"/80/foo/bar":        {"", "80", "foo/bar", false},
+		"service//foo/bar":   {"service", "", "foo/bar", false},
+		"service/80/foo":     {"service", "80", "foo", true},
+		"service/80/foo/bar": {"service", "80", "foo/bar", true},
+	}
+
+	for tn, tc := range tests {
+		t.Run(tn, func(t *testing.T) {
+			service, port, requestedURI, ok := parseProxyParams(tn)
+			assert.Equal(t, tc.service, service)
+			assert.Equal(t, tc.port, port)
+			assert.Equal(t, tc.requestedURI, requestedURI)
+			assert.Equal(t, tc.ok, ok)
+		})
+	}
+}
+
 func mockProxy(
 	serviceName string,
 	port int,
