@@ -1,8 +1,11 @@
 package shells
 
-import "gitlab.com/gitlab-org/gitlab-runner/common"
+import (
+	"gitlab.com/gitlab-org/gitlab-runner/common"
+)
 
 type ShellWriter interface {
+	EnvVariableKey(name string) string
 	Variable(variable common.JobVariable)
 	Command(command string, arguments ...string)
 	Line(text string)
@@ -19,14 +22,21 @@ type ShellWriter interface {
 	MkDir(path string)
 	RmDir(path string)
 	RmFile(path string)
+	RmFilesRecursive(path string, name string)
 	Absolute(path string) string
+	Join(elem ...string) string
 	TmpFile(name string) string
 
 	MkTmpDir(name string) string
 
-	Print(fmt string, arguments ...interface{})
-	Notice(fmt string, arguments ...interface{})
-	Warning(fmt string, arguments ...interface{})
-	Error(fmt string, arguments ...interface{})
+	Printf(fmt string, arguments ...interface{})
+	Noticef(fmt string, arguments ...interface{})
+	Warningf(fmt string, arguments ...interface{})
+	Errorf(fmt string, arguments ...interface{})
 	EmptyLine()
+
+	SectionStart(id, command string)
+	SectionEnd(id string)
+
+	Finish(trace bool) string
 }
